"""
This type stub file was generated by pyright.
"""

from typing import Union as UnionType
from IPython.core.interactiveshell import InteractiveShell
from traitlets import Integer, observe
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.history import History
from prompt_toolkit.shortcuts import PromptSession
from pygments.style import Style
from .shortcuts.auto_suggest import NavigableAutoSuggestFromHistory

"""IPython terminal interface using prompt_toolkit"""
PTK3 = ...
class _NoStyle(Style):
    ...


_style_overrides_light_bg = ...
_style_overrides_linux = ...
def get_default_editor():
    ...

_use_simple_prompt = ...
def black_reformat_handler(text_before_cursor):
    """
    We do not need to protect against error,
    this is taken care at a higher level where any reformat error is ignored.
    Indeed we may call reformatting on incomplete code.
    """
    ...

def yapf_reformat_handler(text_before_cursor):
    ...

class PtkHistoryAdapter(History):
    """
    Prompt toolkit has it's own way of handling history, Where it assumes it can
    Push/pull from history.

    """
    def __init__(self, shell) -> None:
        ...
    
    def append_string(self, string): # -> None:
        ...
    
    def load_history_strings(self):
        ...
    
    def store_string(self, string: str) -> None:
        ...
    


class TerminalInteractiveShell(InteractiveShell):
    mime_renderers = ...
    space_for_menu = Integer(6, help='Number of line at the bottom of the screen ' 'to reserve for the tab completion menu, ' 'search history, ...etc, the height of ' 'these menus will at most this value. ' 'Increase it is you prefer long and skinny ' 'menus, decrease for short and wide.').tag(config=True)
    pt_app: UnionType[PromptSession, None] = ...
    auto_suggest: UnionType[AutoSuggestFromHistory, NavigableAutoSuggestFromHistory, None] = ...
    debugger_history = ...
    debugger_history_file = ...
    simple_prompt = ...
    @property
    def debugger_cls(self): # -> Type[Pdb] | Type[TerminalPdb]:
        ...
    
    confirm_exit = ...
    editing_mode = ...
    emacs_bindings_in_vi_insert_mode = ...
    modal_cursor = ...
    ttimeoutlen = ...
    timeoutlen = ...
    autoformatter = ...
    auto_match = ...
    mouse_support = ...
    highlighting_style = ...
    def refresh_style(self): # -> None:
        ...
    
    highlighting_style_overrides = ...
    true_color = ...
    editor = ...
    prompts_class = ...
    prompts = ...
    term_title = ...
    term_title_format = ...
    display_completions = ...
    highlight_matching_brackets = ...
    extra_open_editor_shortcuts = ...
    handle_return = ...
    enable_history_search = ...
    autosuggestions_provider = ...
    shortcuts = ...
    prompt_includes_vi_mode = ...
    @observe('term_title')
    def init_term_title(self, change=...): # -> None:
        ...
    
    def restore_term_title(self): # -> None:
        ...
    
    def init_display_formatter(self): # -> None:
        ...
    
    def init_prompt_toolkit_cli(self): # -> None:
        ...
    
    @property
    def pt_complete_style(self):
        ...
    
    @property
    def color_depth(self): # -> Literal[ColorDepth.TRUE_COLOR] | None:
        ...
    
    def prompt_for_code(self):
        ...
    
    def enable_win_unicode_console(self): # -> None:
        ...
    
    def init_io(self): # -> None:
        ...
    
    def init_magics(self): # -> None:
        ...
    
    def init_alias(self): # -> None:
        ...
    
    def __init__(self, *args, **kwargs) -> None:
        ...
    
    def ask_exit(self): # -> None:
        ...
    
    rl_next_input = ...
    def interact(self): # -> None:
        ...
    
    def mainloop(self): # -> None:
        ...
    
    _inputhook = ...
    def inputhook(self, context): # -> None:
        ...
    
    active_eventloop = ...
    def enable_gui(self, gui=...): # -> None:
        ...
    
    system = ...
    def auto_rewrite_input(self, cmd): # -> None:
        """Overridden from the parent class to use fancy rewriting prompt"""
        ...
    
    _prompts_before = ...
    def switch_doctest_mode(self, mode): # -> None:
        """Switch prompts to classic for %doctest_mode"""
        ...
    


if __name__ == '__main__':
    ...



from typing import Callable, Literal, Sequence, TypeVar
from ..base import BaseEstimator
from numpy import ndarray
from ..base import ClassifierMixin
from .validation import check_array as check_array, check_is_fitted as check_is_fitted
from .._typing import Int, MatrixLike, ArrayLike

CheckingClassifier_Self = TypeVar("CheckingClassifier_Self", bound="CheckingClassifier")

import numpy as np


class ArraySlicingWrapper:
    def __init__(self, array) -> None:
        ...

    def __getitem__(self, aslice):
        ...


class MockDataFrame:

    # have shape and length but don't support indexing.

    def __init__(self, array) -> None:
        ...

    def __len__(self) -> int:
        ...

    def __array__(self, dtype=None):
        ...

    def __eq__(self, other) -> bool:
        ...

    def __ne__(self, other) -> bool:
        ...

    def take(self, indices, axis: int = 0):
        ...


class CheckingClassifier(ClassifierMixin, BaseEstimator):
    n_features_in_: int = ...
    classes_: int = ...

    def __init__(
        self,
        *,
        check_y: None | Callable = None,
        check_y_params: None | dict = None,
        check_X: None | Callable = None,
        check_X_params: None | dict = None,
        methods_to_check: Literal["all", "all"] | Sequence[str] = "all",
        foo_param: Int = 0,
        expected_sample_weight: None | bool = None,
        expected_fit_params: None | Sequence[str] = None,
    ) -> None:
        ...

    def fit(
        self: CheckingClassifier_Self,
        X: MatrixLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
        **fit_params,
    ) -> CheckingClassifier_Self:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...

    def decision_function(self, X: MatrixLike) -> ndarray:
        ...

    def score(
        self, X: None | MatrixLike = None, Y: None | MatrixLike | ArrayLike = None
    ) -> float:
        ...


class NoSampleWeightWrapper(BaseEstimator):
    def __init__(self, est: None | BaseEstimator = None) -> None:
        ...

    def fit(self, X, y):
        ...

    def predict(self, X):
        ...

    def predict_proba(self, X):
        ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.signal._spectral, version: unspecified
import typing
import builtins as _mod_builtins

__all__: list
__doc__: str
__file__: str
__name__: str
__package__: str
__test__: dict
def _lombscargle(x, y, freqs) -> typing.Any:
    '\n    _lombscargle(x, y, freqs)\n\n    Computes the Lomb-Scargle periodogram.\n\n    Parameters\n    ----------\n    x : array_like\n        Sample times.\n    y : array_like\n        Measurement values (must be registered so the mean is zero).\n    freqs : array_like\n        Angular frequencies for output periodogram.\n\n    Returns\n    -------\n    pgram : array_like\n        Lomb-Scargle periodogram.\n\n    Raises\n    ------\n    ValueError\n        If the input arrays `x` and `y` do not have the same shape.\n\n    See also\n    --------\n    lombscargle\n\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




magic = ...
align_1_checker_value: bytes = ...
align_1_offset: int = ...
align_1_length: int = ...
align_1_value: int = ...
u64_byte_checker_value: bytes = ...
align_2_offset: int = ...
align_2_length: int = ...
align_2_value: int = ...
endianness_offset: int = ...
endianness_length: int = ...
platform_offset: int = ...
platform_length: int = ...
encoding_offset: int = ...
encoding_length: int = ...
dataset_offset: int = ...
dataset_length: int = ...
file_type_offset: int = ...
file_type_length: int = ...
date_created_offset: int = ...
date_created_length: int = ...
date_modified_offset: int = ...
date_modified_length: int = ...
header_size_offset: int = ...
header_size_length: int = ...
page_size_offset: int = ...
page_size_length: int = ...
page_count_offset: int = ...
page_count_length: int = ...
sas_release_offset: int = ...
sas_release_length: int = ...
sas_server_type_offset: int = ...
sas_server_type_length: int = ...
os_version_number_offset: int = ...
os_version_number_length: int = ...
os_maker_offset: int = ...
os_maker_length: int = ...
os_name_offset: int = ...
os_name_length: int = ...
page_bit_offset_x86: int = ...
page_bit_offset_x64: int = ...
subheader_pointer_length_x86: int = ...
subheader_pointer_length_x64: int = ...
page_type_offset: int = ...
page_type_length: int = ...
block_count_offset: int = ...
block_count_length: int = ...
subheader_count_offset: int = ...
subheader_count_length: int = ...
page_meta_type: int = ...
page_data_type: int = ...
page_amd_type: int = ...
page_metc_type: int = ...
page_comp_type: int = ...
page_mix_types = ...
subheader_pointers_offset: int = ...
truncated_subheader_id: int = ...
compressed_subheader_id: int = ...
compressed_subheader_type: int = ...
text_block_size_length: int = ...
row_length_offset_multiplier: int = ...
row_count_offset_multiplier: int = ...
col_count_p1_multiplier: int = ...
col_count_p2_multiplier: int = ...
row_count_on_mix_page_offset_multiplier: int = ...
column_name_pointer_length: int = ...
column_name_text_subheader_offset: int = ...
column_name_text_subheader_length: int = ...
column_name_offset_offset: int = ...
column_name_offset_length: int = ...
column_name_length_offset: int = ...
column_name_length_length: int = ...
column_data_offset_offset: int = ...
column_data_length_offset: int = ...
column_data_length_length: int = ...
column_type_offset: int = ...
column_type_length: int = ...
column_format_text_subheader_index_offset: int = ...
column_format_text_subheader_index_length: int = ...
column_format_offset_offset: int = ...
column_format_offset_length: int = ...
column_format_length_offset: int = ...
column_format_length_length: int = ...
column_label_text_subheader_index_offset: int = ...
column_label_text_subheader_index_length: int = ...
column_label_offset_offset: int = ...
column_label_offset_length: int = ...
column_label_length_offset: int = ...
column_label_length_length: int = ...
rle_compression: bytes = ...
rdc_compression: bytes = ...
compression_literals = ...
encoding_names = ...

class SASIndex:
    row_size_index: int = ...
    column_size_index: int = ...
    subheader_counts_index: int = ...
    column_text_index: int = ...
    column_name_index: int = ...
    column_attributes_index: int = ...
    format_and_label_index: int = ...
    column_list_index: int = ...
    data_subheader_index: int = ...

subheader_signature_to_index = ...
sas_date_formats = ...
sas_datetime_formats = ...



from numpy.typing import ArrayLike
from typing import Sequence

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from .visual import CompoundVisual
from .mesh import MeshVisual
from .line import LineVisual
from ..color import Color
from ..geometry import PolygonData

class PolygonVisual(CompoundVisual):
    def __init__(
        self,
        pos: ArrayLike | None = None,
        color: str | tuple | Sequence = "black",
        border_color: str | tuple | Sequence | None = None,
        border_width: int = 1,
        border_method: str = "gl",
        triangulate: bool = True,
        **kwargs,
    ): ...
    def _update(self): ...
    @property
    def pos(self): ...
    @pos.setter
    def pos(self, pos): ...
    @property
    def color(self): ...
    @color.setter
    def color(self, color): ...
    @property
    def border_color(self): ...
    @border_color.setter
    def border_color(self, border_color): ...
    @property
    def mesh(self): ...
    @mesh.setter
    def mesh(self, mesh): ...
    @property
    def border(self): ...
    @border.setter
    def border(self, border): ...



from collections.abc import (
    Callable,
    Hashable,
    Iterable,
    Mapping,
    Sequence,
)
from types import TracebackType
from typing import (
    Any,
    Literal,
    overload,
)

from odf.opendocument import OpenDocument
from openpyxl.workbook.workbook import Workbook
from pandas.core.frame import DataFrame
import pyxlsb.workbook
from typing_extensions import Self
from xlrd.book import Book

from pandas._libs.lib import NoDefault
from pandas._typing import (
    Dtype,
    DtypeBackend,
    FilePath,
    ListLikeHashable,
    ReadBuffer,
    StorageOptions,
    UsecolsArgType,
    WriteExcelBuffer,
)

@overload
def read_excel(
    io: FilePath
    | ReadBuffer[bytes]
    | bytes
    | ExcelFile
    | Workbook
    | Book
    | OpenDocument
    | pyxlsb.workbook.Workbook,
    sheet_name: list[int | str] | None,
    *,
    header: int | Sequence[int] | None = ...,
    names: ListLikeHashable | None = ...,
    index_col: int | Sequence[int] | None = ...,
    usecols: str | UsecolsArgType = ...,
    dtype: str | Dtype | Mapping[str, str | Dtype] | None = ...,
    engine: Literal["xlrd", "openpyxl", "odf", "pyxlsb"] | None = ...,
    converters: Mapping[int | str, Callable[[object], object]] | None = ...,
    true_values: Iterable[Hashable] | None = ...,
    false_values: Iterable[Hashable] | None = ...,
    skiprows: int | Sequence[int] | Callable[[object], bool] | None = ...,
    nrows: int | None = ...,
    na_values: Sequence[str] | dict[str | int, Sequence[str]] = ...,
    keep_default_na: bool = ...,
    na_filter: bool = ...,
    verbose: bool = ...,
    parse_dates: bool
    | Sequence[int]
    | Sequence[Sequence[str] | Sequence[int]]
    | dict[str, Sequence[int] | list[str]] = ...,
    date_format: dict[Hashable, str] | str | None = ...,
    thousands: str | None = ...,
    decimal: str = ...,
    comment: str | None = ...,
    skipfooter: int = ...,
    storage_options: StorageOptions = ...,
    dtype_backend: DtypeBackend | NoDefault = ...,
) -> dict[int | str, DataFrame]:
    """
Read an Excel file into a pandas DataFrame.

Supports `xls`, `xlsx`, `xlsm`, `xlsb`, `odf`, `ods` and `odt` file extensions
read from a local filesystem or URL. Supports an option to read
a single sheet or a list of sheets.

Parameters
----------
io : str, bytes, ExcelFile, xlrd.Book, path object, or file-like object
    Any valid string path is acceptable. The string could be a URL. Valid
    URL schemes include http, ftp, s3, and file. For file URLs, a host is
    expected. A local file could be: ``file://localhost/path/to/table.xlsx``.

    If you want to pass in a path object, pandas accepts any ``os.PathLike``.

    By file-like object, we refer to objects with a ``read()`` method,
    such as a file handle (e.g. via builtin ``open`` function)
    or ``StringIO``.
sheet_name : str, int, list, or None, default 0
    Strings are used for sheet names. Integers are used in zero-indexed
    sheet positions (chart sheets do not count as a sheet position).
    Lists of strings/integers are used to request multiple sheets.
    Specify None to get all worksheets.

    Available cases:

    * Defaults to ``0``: 1st sheet as a `DataFrame`
    * ``1``: 2nd sheet as a `DataFrame`
    * ``"Sheet1"``: Load sheet with name "Sheet1"
    * ``[0, 1, "Sheet5"]``: Load first, second and sheet named "Sheet5"
      as a dict of `DataFrame`
    * None: All worksheets.

header : int, list of int, default 0
    Row (0-indexed) to use for the column labels of the parsed
    DataFrame. If a list of integers is passed those row positions will
    be combined into a ``MultiIndex``. Use None if there is no header.
names : array-like, default None
    List of column names to use. If file contains no header row,
    then you should explicitly pass header=None.
index_col : int, list of int, default None
    Column (0-indexed) to use as the row labels of the DataFrame.
    Pass None if there is no such column.  If a list is passed,
    those columns will be combined into a ``MultiIndex``.  If a
    subset of data is selected with ``usecols``, index_col
    is based on the subset.

    Missing values will be forward filled to allow roundtripping with
    ``to_excel`` for ``merged_cells=True``. To avoid forward filling the
    missing values use ``set_index`` after reading the data instead of
    ``index_col``.
usecols : str, list-like, or callable, default None
    * If None, then parse all columns.
    * If str, then indicates comma separated list of Excel column letters
      and column ranges (e.g. "A:E" or "A,C,E:F"). Ranges are inclusive of
      both sides.
    * If list of int, then indicates list of column numbers to be parsed
      (0-indexed).
    * If list of string, then indicates list of column names to be parsed.
    * If callable, then evaluate each column name against it and parse the
      column if the callable returns ``True``.

    Returns a subset of the columns according to behavior above.
dtype : Type name or dict of column -> type, default None
    Data type for data or columns. E.g. {'a': np.float64, 'b': np.int32}
    Use `object` to preserve data as stored in Excel and not interpret dtype.
    If converters are specified, they will be applied INSTEAD
    of dtype conversion.
engine : str, default None
    If io is not a buffer or path, this must be set to identify io.
    Supported engines: "xlrd", "openpyxl", "odf", "pyxlsb".
    Engine compatibility :

    - "xlrd" supports old-style Excel files (.xls).
    - "openpyxl" supports newer Excel file formats.
    - "odf" supports OpenDocument file formats (.odf, .ods, .odt).
    - "pyxlsb" supports Binary Excel files.

    .. versionchanged:: 1.2.0
        The engine `xlrd <https://xlrd.readthedocs.io/en/latest/>`_
        now only supports old-style ``.xls`` files.
        When ``engine=None``, the following logic will be
        used to determine the engine:

       - If ``path_or_buffer`` is an OpenDocument format (.odf, .ods, .odt),
         then `odf <https://pypi.org/project/odfpy/>`_ will be used.
       - Otherwise if ``path_or_buffer`` is an xls format,
         ``xlrd`` will be used.
       - Otherwise if ``path_or_buffer`` is in xlsb format,
         ``pyxlsb`` will be used.

         .. versionadded:: 1.3.0
       - Otherwise ``openpyxl`` will be used.

         .. versionchanged:: 1.3.0

converters : dict, default None
    Dict of functions for converting values in certain columns. Keys can
    either be integers or column labels, values are functions that take one
    input argument, the Excel cell content, and return the transformed
    content.
true_values : list, default None
    Values to consider as True.
false_values : list, default None
    Values to consider as False.
skiprows : list-like, int, or callable, optional
    Line numbers to skip (0-indexed) or number of lines to skip (int) at the
    start of the file. If callable, the callable function will be evaluated
    against the row indices, returning True if the row should be skipped and
    False otherwise. An example of a valid callable argument would be ``lambda
    x: x in [0, 2]``.
nrows : int, default None
    Number of rows to parse.
na_values : scalar, str, list-like, or dict, default None
    Additional strings to recognize as NA/NaN. If dict passed, specific
    per-column NA values. By default the following values are interpreted
    as NaN: '', '#N/A', '#N/A N/A', '#NA', '-1.#IND', '-1.#QNAN', '-NaN', '-nan',
    '1.#IND', '1.#QNAN', '<NA>', 'N/A', 'NA', 'NULL', 'NaN', 'None',
    'n/a', 'nan', 'null'.
keep_default_na : bool, default True
    Whether or not to include the default NaN values when parsing the data.
    Depending on whether `na_values` is passed in, the behavior is as follows:

    * If `keep_default_na` is True, and `na_values` are specified, `na_values`
      is appended to the default NaN values used for parsing.
    * If `keep_default_na` is True, and `na_values` are not specified, only
      the default NaN values are used for parsing.
    * If `keep_default_na` is False, and `na_values` are specified, only
      the NaN values specified `na_values` are used for parsing.
    * If `keep_default_na` is False, and `na_values` are not specified, no
      strings will be parsed as NaN.

    Note that if `na_filter` is passed in as False, the `keep_default_na` and
    `na_values` parameters will be ignored.
na_filter : bool, default True
    Detect missing value markers (empty strings and the value of na_values). In
    data without any NAs, passing na_filter=False can improve the performance
    of reading a large file.
verbose : bool, default False
    Indicate number of NA values placed in non-numeric columns.
parse_dates : bool, list-like, or dict, default False
    The behavior is as follows:

    * bool. If True -> try parsing the index.
    * list of int or names. e.g. If [1, 2, 3] -> try parsing columns 1, 2, 3
      each as a separate date column.
    * list of lists. e.g.  If [[1, 3]] -> combine columns 1 and 3 and parse as
      a single date column.
    * dict, e.g. {'foo' : [1, 3]} -> parse columns 1, 3 as date and call
      result 'foo'

    If a column or index contains an unparsable date, the entire column or
    index will be returned unaltered as an object data type. If you don`t want to
    parse some cells as date just change their type in Excel to "Text".
    For non-standard datetime parsing, use ``pd.to_datetime`` after ``pd.read_excel``.

    Note: A fast-path exists for iso8601-formatted dates.
date_parser : function, optional
    Function to use for converting a sequence of string columns to an array of
    datetime instances. The default uses ``dateutil.parser.parser`` to do the
    conversion. Pandas will try to call `date_parser` in three different ways,
    advancing to the next if an exception occurs: 1) Pass one or more arrays
    (as defined by `parse_dates`) as arguments; 2) concatenate (row-wise) the
    string values from the columns defined by `parse_dates` into a single array
    and pass that; and 3) call `date_parser` once for each row using one or
    more strings (corresponding to the columns defined by `parse_dates`) as
    arguments.

    .. deprecated:: 2.0.0
       Use ``date_format`` instead, or read in as ``object`` and then apply
       :func:`to_datetime` as-needed.
date_format : str or dict of column -> format, default ``None``
   If used in conjunction with ``parse_dates``, will parse dates according to this
   format. For anything more complex,
   please read in as ``object`` and then apply :func:`to_datetime` as-needed.

   .. versionadded:: 2.0.0
thousands : str, default None
    Thousands separator for parsing string columns to numeric.  Note that
    this parameter is only necessary for columns stored as TEXT in Excel,
    any numeric columns will automatically be parsed, regardless of display
    format.
decimal : str, default '.'
    Character to recognize as decimal point for parsing string columns to numeric.
    Note that this parameter is only necessary for columns stored as TEXT in Excel,
    any numeric columns will automatically be parsed, regardless of display
    format.(e.g. use ',' for European data).

    .. versionadded:: 1.4.0

comment : str, default None
    Comments out remainder of line. Pass a character or characters to this
    argument to indicate comments in the input file. Any data between the
    comment string and the end of the current line is ignored.
skipfooter : int, default 0
    Rows at the end to skip (0-indexed).
storage_options : dict, optional
    Extra options that make sense for a particular storage connection, e.g.
    host, port, username, password, etc. For HTTP(S) URLs the key-value pairs
    are forwarded to ``urllib.request.Request`` as header options. For other
    URLs (e.g. starting with "s3://", and "gcs://") the key-value pairs are
    forwarded to ``fsspec.open``. Please see ``fsspec`` and ``urllib`` for more
    details, and for more examples on storage options refer `here
    <https://pandas.pydata.org/docs/user_guide/io.html?
    highlight=storage_options#reading-writing-remote-files>`_.

    .. versionadded:: 1.2.0

dtype_backend : {"numpy_nullable", "pyarrow"}, defaults to NumPy backed DataFrames
    Which dtype_backend to use, e.g. whether a DataFrame should have NumPy
    arrays, nullable dtypes are used for all dtypes that have a nullable
    implementation when "numpy_nullable" is set, pyarrow is used for all
    dtypes if "pyarrow" is set.

    The dtype_backends are still experimential.

    .. versionadded:: 2.0

Returns
-------
DataFrame or dict of DataFrames
    DataFrame from the passed in Excel file. See notes in sheet_name
    argument for more information on when a dict of DataFrames is returned.

See Also
--------
DataFrame.to_excel : Write DataFrame to an Excel file.
DataFrame.to_csv : Write DataFrame to a comma-separated values (csv) file.
read_csv : Read a comma-separated values (csv) file into DataFrame.
read_fwf : Read a table of fixed-width formatted lines into DataFrame.

Examples
--------
The file can be read using the file name as string or an open file object:

>>> pd.read_excel('tmp.xlsx', index_col=0)  # doctest: +SKIP
       Name  Value
0   string1      1
1   string2      2
2  #Comment      3

>>> pd.read_excel(open('tmp.xlsx', 'rb'),
...               sheet_name='Sheet3')  # doctest: +SKIP
   Unnamed: 0      Name  Value
0           0   string1      1
1           1   string2      2
2           2  #Comment      3

Index and header can be specified via the `index_col` and `header` arguments

>>> pd.read_excel('tmp.xlsx', index_col=None, header=None)  # doctest: +SKIP
     0         1      2
0  NaN      Name  Value
1  0.0   string1      1
2  1.0   string2      2
3  2.0  #Comment      3

Column types are inferred but can be explicitly specified

>>> pd.read_excel('tmp.xlsx', index_col=0,
...               dtype={'Name': str, 'Value': float})  # doctest: +SKIP
       Name  Value
0   string1    1.0
1   string2    2.0
2  #Comment    3.0

True, False, and NA values, and thousands separators have defaults,
but can be explicitly specified, too. Supply the values you would like
as strings or lists of strings!

>>> pd.read_excel('tmp.xlsx', index_col=0,
...               na_values=['string1', 'string2'])  # doctest: +SKIP
       Name  Value
0       NaN      1
1       NaN      2
2  #Comment      3

Comment lines in the excel input file can be skipped using the `comment` kwarg

>>> pd.read_excel('tmp.xlsx', index_col=0, comment='#')  # doctest: +SKIP
      Name  Value
0  string1    1.0
1  string2    2.0
2     None    NaN
    """
    pass
@overload
def read_excel(
    io: FilePath
    | ReadBuffer[bytes]
    | bytes
    | ExcelFile
    | Workbook
    | Book
    | OpenDocument
    | pyxlsb.workbook.Workbook,
    sheet_name: int | str = ...,
    *,
    header: int | Sequence[int] | None = ...,
    names: ListLikeHashable | None = ...,
    index_col: int | Sequence[int] | None = ...,
    usecols: str | UsecolsArgType = ...,
    dtype: str | Dtype | Mapping[str, str | Dtype] | None = ...,
    engine: Literal["xlrd", "openpyxl", "odf", "pyxlsb"] | None = ...,
    converters: Mapping[int | str, Callable[[object], object]] | None = ...,
    true_values: Iterable[Hashable] | None = ...,
    false_values: Iterable[Hashable] | None = ...,
    skiprows: int | Sequence[int] | Callable[[object], bool] | None = ...,
    nrows: int | None = ...,
    na_values: Sequence[str] | dict[str | int, Sequence[str]] = ...,
    keep_default_na: bool = ...,
    na_filter: bool = ...,
    verbose: bool = ...,
    parse_dates: bool
    | Sequence[int]
    | Sequence[Sequence[str] | Sequence[int]]
    | dict[str, Sequence[int] | list[str]] = ...,
    date_format: dict[Hashable, str] | str | None = ...,
    thousands: str | None = ...,
    decimal: str = ...,
    comment: str | None = ...,
    skipfooter: int = ...,
    storage_options: StorageOptions = ...,
    dtype_backend: DtypeBackend | NoDefault = ...,
) -> DataFrame: ...

class ExcelWriter:
    def __init__(
        self,
        path: FilePath | WriteExcelBuffer | ExcelWriter,
        engine: Literal["auto", "openpyxl", "odf", "xlsxwriter"] | None = ...,
        date_format: str | None = ...,
        datetime_format: str | None = ...,
        mode: Literal["w", "a"] = ...,
        storage_options: StorageOptions = ...,
        if_sheet_exists: Literal["error", "new", "replace", "overlay"] | None = ...,
        engine_kwargs: dict[str, Any] | None = ...,
    ) -> None: ...
    @property
    def supported_extensions(self) -> tuple[str, ...]: ...
    @property
    def engine(self) -> Literal["openpyxl", "odf", "xlsxwriter"]: ...
    @property
    def sheets(self) -> dict[str, Any]: ...
    @property
    def book(self) -> Workbook | OpenDocument: ...
    @property
    def date_format(self) -> str: ...
    @property
    def datetime_format(self) -> str: ...
    @property
    def if_sheet_exists(self) -> Literal["error", "new", "replace", "overlay"]: ...
    def __fspath__(self) -> str: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        traceback: TracebackType | None,
    ) -> None: ...
    def close(self) -> None: ...

class ExcelFile:
    engine = ...
    io: FilePath | ReadBuffer[bytes] | bytes = ...
    def __init__(
        self,
        io: FilePath | ReadBuffer[bytes] | bytes,
        engine: Literal["xlrd", "openpyxl", "odf", "pyxlsb"] | None = ...,
        storage_options: StorageOptions = ...,
    ) -> None: ...
    def __fspath__(self): ...
    @overload
    def parse(
        self,
        sheet_name: list[int | str] | None,
        header: int | Sequence[int] | None = ...,
        names: ListLikeHashable | None = ...,
        index_col: int | Sequence[int] | None = ...,
        usecols: str | UsecolsArgType = ...,
        converters: dict[int | str, Callable[[object], object]] | None = ...,
        true_values: Iterable[Hashable] | None = ...,
        false_values: Iterable[Hashable] | None = ...,
        skiprows: int | Sequence[int] | Callable[[object], bool] | None = ...,
        nrows: int | None = ...,
        na_values: Sequence[str] | dict[str | int, Sequence[str]] = ...,
        parse_dates: bool
        | Sequence[int]
        | Sequence[Sequence[str] | Sequence[int]]
        | dict[str, Sequence[int] | list[str]] = ...,
        date_parser: Callable | None = ...,
        thousands: str | None = ...,
        comment: str | None = ...,
        skipfooter: int = ...,
        keep_default_na: bool = ...,
        na_filter: bool = ...,
        **kwds: Any,
    ) -> dict[int | str, DataFrame]: ...
    @overload
    def parse(
        self,
        sheet_name: int | str,
        header: int | Sequence[int] | None = ...,
        names: ListLikeHashable | None = ...,
        index_col: int | Sequence[int] | None = ...,
        usecols: str | UsecolsArgType = ...,
        converters: dict[int | str, Callable[[object], object]] | None = ...,
        true_values: Iterable[Hashable] | None = ...,
        false_values: Iterable[Hashable] | None = ...,
        skiprows: int | Sequence[int] | Callable[[object], bool] | None = ...,
        nrows: int | None = ...,
        na_values: Sequence[str] | dict[str | int, Sequence[str]] = ...,
        parse_dates: bool
        | Sequence[int]
        | Sequence[Sequence[str] | Sequence[int]]
        | dict[str, Sequence[int] | list[str]] = ...,
        date_parser: Callable | None = ...,
        thousands: str | None = ...,
        comment: str | None = ...,
        skipfooter: int = ...,
        keep_default_na: bool = ...,
        na_filter: bool = ...,
        **kwds: Any,
    ) -> DataFrame: ...
    @property
    def book(self) -> Workbook | Book | OpenDocument | pyxlsb.workbook.Workbook: ...
    @property
    def sheet_names(self) -> list[int | str]: ...
    def close(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        traceback: TracebackType | None,
    ) -> None: ...
    def __del__(self) -> None: ...



from collections.abc import Iterable
from numpy.typing import ArrayLike
from ...exception import NetworkXAlgorithmError
from ...utils import not_implemented_for

__all__ = [
    "project",
    "projected_graph",
    "weighted_projected_graph",
    "collaboration_weighted_projected_graph",
    "overlap_weighted_projected_graph",
    "generic_weighted_projected_graph",
]

def projected_graph(B, nodes: ArrayLike | Iterable, multigraph=False): ...
def weighted_projected_graph(B, nodes: ArrayLike | Iterable, ratio=False): ...
def collaboration_weighted_projected_graph(B, nodes: ArrayLike | Iterable): ...
def overlap_weighted_projected_graph(B, nodes: ArrayLike | Iterable, jaccard=True): ...
def generic_weighted_projected_graph(
    B, nodes: ArrayLike | Iterable, weight_function=None
): ...
def project(B, nodes, create_using=None): ...



from typing import Any, Callable
import warnings
import functools


__all__ = ["deprecated"]


class deprecated:

    # Adapted from https://wiki.python.org/moin/PythonDecoratorLibrary,
    # but with many changes.

    def __init__(self, extra: str = "") -> None:
        ...

    def __call__(self, obj: Any) -> property | Callable:
        ...



import numpy as np
import scipy.ndimage as ndi

from .._shared.filters import gaussian
from .._shared.utils import check_nD

def _preprocess(image, mask, sigma, mode, cval): ...
def _set_local_maxima(magnitude, pts, w_num, w_denum, row_slices, col_slices, out): ...
def _get_local_maxima(isobel, jsobel, magnitude, eroded_mask): ...
def canny(
    image,
    sigma: float = 1.0,
    low_threshold: float | None = None,
    high_threshold: float | None = None,
    mask=None,
    use_quantiles: bool = False,
    *,
    mode="constant",
    cval: float = 0.0
): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.sparse.csgraph._shortest_path, version: unspecified
import typing
import builtins as _mod_builtins
import numpy as _mod_numpy
import scipy.sparse.csr as _mod_scipy_sparse_csr

DTYPE = _mod_numpy.float64
ITYPE = _mod_numpy.int32
class NegativeCycleError(_mod_builtins.Exception):
    __dict__: typing.Dict[str, typing.Any]
    def __init__(self, message) -> None:
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __module__: str
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    @property
    def __weakref__(self) -> typing.Any:
        'list of weak references to the object (if defined)'
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

__doc__: str
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def bellman_ford(csgraph, directed=..., indices=..., return_predecessors=..., unweighted=...) -> typing.Any:
    "\n    bellman_ford(csgraph, directed=True, indices=None, return_predecessors=False,\n                 unweighted=False)\n\n    Compute the shortest path lengths using the Bellman-Ford algorithm.\n\n    The Bellman-Ford algorithm can robustly deal with graphs with negative\n    weights.  If a negative cycle is detected, an error is raised.  For\n    graphs without negative edge weights, Dijkstra's algorithm may be faster.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array, matrix, or sparse matrix, 2 dimensions\n        The N x N array of distances representing the input graph.\n    directed : bool, optional\n        If True (default), then find the shortest path on a directed graph:\n        only move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i]\n    indices : array_like or int, optional\n        if specified, only compute the paths from the points at the given\n        indices.\n    return_predecessors : bool, optional\n        If True, return the size (N, N) predecesor matrix\n    unweighted : bool, optional\n        If True, then find unweighted distances.  That is, rather than finding\n        the path between each point such that the sum of weights is minimized,\n        find the path such that the number of edges is minimized.\n\n    Returns\n    -------\n    dist_matrix : ndarray\n        The N x N matrix of distances between graph nodes. dist_matrix[i,j]\n        gives the shortest distance from point i to point j along the graph.\n\n    predecessors : ndarray\n        Returned only if return_predecessors == True.\n        The N x N matrix of predecessors, which can be used to reconstruct\n        the shortest paths.  Row i of the predecessor matrix contains\n        information on the shortest paths from point i: each entry\n        predecessors[i, j] gives the index of the previous node in the\n        path from point i to point j.  If no path exists between point\n        i and j, then predecessors[i, j] = -9999\n\n    Raises\n    ------\n    NegativeCycleError:\n        if there are negative cycles in the graph\n\n    Notes\n    -----\n    This routine is specially designed for graphs with negative edge weights.\n    If all edge weights are positive, then Dijkstra's algorithm is a better\n    choice.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import bellman_ford\n\n    >>> graph = [\n    ... [0, 1 ,2, 0],\n    ... [0, 0, 0, 1],\n    ... [2, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)\t1\n      (0, 2)\t2\n      (1, 3)\t1\n      (2, 0)\t2\n      (2, 3)\t3\n\n    >>> dist_matrix, predecessors = bellman_ford(csgraph=graph, directed=False, indices=0, return_predecessors=True)\n    >>> dist_matrix\n    array([ 0.,  1.,  2.,  2.])\n    >>> predecessors\n    array([-9999,     0,     0,     1], dtype=int32)\n\n    "
    ...

csr_matrix = _mod_scipy_sparse_csr.csr_matrix
def dijkstra(csgraph, directed=..., indices=..., return_predecessors=..., unweighted=..., limit=..., min_only=...) -> typing.Any:
    "\n    dijkstra(csgraph, directed=True, indices=None, return_predecessors=False,\n             unweighted=False, limit=np.inf, min_only=False)\n\n    Dijkstra algorithm using Fibonacci Heaps\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array, matrix, or sparse matrix, 2 dimensions\n        The N x N array of non-negative distances representing the input graph.\n    directed : bool, optional\n        If True (default), then find the shortest path on a directed graph:\n        only move from point i to point j along paths csgraph[i, j] and from\n        point j to i along paths csgraph[j, i].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j or j to i along either\n        csgraph[i, j] or csgraph[j, i].\n    indices : array_like or int, optional\n        if specified, only compute the paths from the points at the given\n        indices.\n    return_predecessors : bool, optional\n        If True, return the size (N, N) predecesor matrix\n    unweighted : bool, optional\n        If True, then find unweighted distances.  That is, rather than finding\n        the path between each point such that the sum of weights is minimized,\n        find the path such that the number of edges is minimized.\n    limit : float, optional\n        The maximum distance to calculate, must be >= 0. Using a smaller limit\n        will decrease computation time by aborting calculations between pairs\n        that are separated by a distance > limit. For such pairs, the distance\n        will be equal to np.inf (i.e., not connected).\n\n        .. versionadded:: 0.14.0\n    min_only : bool, optional\n        If False (default), for every node in the graph, find the shortest path\n        from every node in indices.\n        If True, for every node in the graph, find the shortest path from any\n        of the nodes in indices (which can be substantially faster).\n\n        .. versionadded:: 1.3.0\n\n    Returns\n    -------\n    dist_matrix : ndarray, shape ([n_indices, ]n_nodes,)\n        The matrix of distances between graph nodes. If min_only=False,\n        dist_matrix has shape (n_indices, n_nodes) and dist_matrix[i, j]\n        gives the shortest distance from point i to point j along the graph.\n        If min_only=True, dist_matrix has shape (n_nodes,) and contains for\n        a given node the shortest path to that node from any of the nodes\n        in indices.\n    predecessors : ndarray, shape ([n_indices, ]n_nodes,)\n        If min_only=False, this has shape (n_indices, n_nodes),\n        otherwise it has shape (n_nodes,).\n        Returned only if return_predecessors == True.\n        The matrix of predecessors, which can be used to reconstruct\n        the shortest paths.  Row i of the predecessor matrix contains\n        information on the shortest paths from point i: each entry\n        predecessors[i, j] gives the index of the previous node in the\n        path from point i to point j.  If no path exists between point\n        i and j, then predecessors[i, j] = -9999\n\n    sources : ndarray, shape (n_nodes,)\n        Returned only if min_only=True and return_predecessors=True.\n        Contains the index of the source which had the shortest path\n        to each target.  If no path exists within the limit,\n        this will contain -9999.  The value at the indices passed\n        will be equal to that index (i.e. the fastest way to reach\n        node i, is to start on node i).\n\n    Notes\n    -----\n    As currently implemented, Dijkstra's algorithm does not work for\n    graphs with direction-dependent distances when directed == False.\n    i.e., if csgraph[i,j] and csgraph[j,i] are not equal and\n    both are nonzero, setting directed=False will not yield the correct\n    result.\n\n    Also, this routine does not work for graphs with negative\n    distances.  Negative distances can lead to infinite cycles that must\n    be handled by specialized algorithms such as Bellman-Ford's algorithm\n    or Johnson's algorithm.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import dijkstra\n\n    >>> graph = [\n    ... [0, 1, 2, 0],\n    ... [0, 0, 0, 1],\n    ... [0, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)\t1\n      (0, 2)\t2\n      (1, 3)\t1\n      (2, 3)\t3\n\n    >>> dist_matrix, predecessors = dijkstra(csgraph=graph, directed=False, indices=0, return_predecessors=True)\n    >>> dist_matrix\n    array([ 0.,  1.,  2.,  2.])\n    >>> predecessors\n    array([-9999,     0,     0,     1], dtype=int32)\n\n    "
    ...

def floyd_warshall(csgraph, directed=..., return_predecessors=..., unweighted=..., overwrite=...) -> typing.Any:
    '\n    floyd_warshall(csgraph, directed=True, return_predecessors=False,\n                   unweighted=False, overwrite=False)\n\n    Compute the shortest path lengths using the Floyd-Warshall algorithm\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array, matrix, or sparse matrix, 2 dimensions\n        The N x N array of distances representing the input graph.\n    directed : bool, optional\n        If True (default), then find the shortest path on a directed graph:\n        only move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i]\n    return_predecessors : bool, optional\n        If True, return the size (N, N) predecesor matrix\n    unweighted : bool, optional\n        If True, then find unweighted distances.  That is, rather than finding\n        the path between each point such that the sum of weights is minimized,\n        find the path such that the number of edges is minimized.\n    overwrite : bool, optional\n        If True, overwrite csgraph with the result.  This applies only if\n        csgraph is a dense, c-ordered array with dtype=float64.\n\n    Returns\n    -------\n    dist_matrix : ndarray\n        The N x N matrix of distances between graph nodes. dist_matrix[i,j]\n        gives the shortest distance from point i to point j along the graph.\n\n    predecessors : ndarray\n        Returned only if return_predecessors == True.\n        The N x N matrix of predecessors, which can be used to reconstruct\n        the shortest paths.  Row i of the predecessor matrix contains\n        information on the shortest paths from point i: each entry\n        predecessors[i, j] gives the index of the previous node in the\n        path from point i to point j.  If no path exists between point\n        i and j, then predecessors[i, j] = -9999\n\n    Raises\n    ------\n    NegativeCycleError:\n        if there are negative cycles in the graph\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import floyd_warshall\n\n    >>> graph = [\n    ... [0, 1 , 2, 0],\n    ... [0, 0, 0, 1],\n    ... [2, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)\t1\n      (0, 2)\t2\n      (1, 3)\t1\n      (2, 0)\t2\n      (2, 3)\t3\n\n\n    >>> dist_matrix, predecessors = floyd_warshall(csgraph=graph, directed=False, return_predecessors=True)\n    >>> dist_matrix\n    array([[ 0.,  1.,  2.,  2.],\n           [ 1.,  0.,  3.,  1.],\n           [ 2.,  3.,  0.,  3.],\n           [ 2.,  1.,  3.,  0.]])\n    >>> predecessors\n    array([[-9999,     0,     0,     1],\n           [    1, -9999,     0,     1],\n           [    2,     0, -9999,     2],\n           [    1,     3,     3, -9999]], dtype=int32)\n\n    '
    ...

def isspmatrix(x) -> typing.Any:
    'Is x of a sparse matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a sparse matrix\n\n    Returns\n    -------\n    bool\n        True if x is a sparse matrix, False otherwise\n\n    Notes\n    -----\n    issparse and isspmatrix are aliases for the same function.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix, isspmatrix\n    >>> isspmatrix(csr_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import isspmatrix\n    >>> isspmatrix(5)\n    False\n    '
    ...

def isspmatrix_csc(x) -> typing.Any:
    'Is x of csc_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a csc matrix\n\n    Returns\n    -------\n    bool\n        True if x is a csc matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import csc_matrix, isspmatrix_csc\n    >>> isspmatrix_csc(csc_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import csc_matrix, csr_matrix, isspmatrix_csc\n    >>> isspmatrix_csc(csr_matrix([[5]]))\n    False\n    '
    ...

def isspmatrix_csr(x) -> typing.Any:
    'Is x of csr_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a csr matrix\n\n    Returns\n    -------\n    bool\n        True if x is a csr matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix, isspmatrix_csr\n    >>> isspmatrix_csr(csr_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import csc_matrix, csr_matrix, isspmatrix_csc\n    >>> isspmatrix_csr(csc_matrix([[5]]))\n    False\n    '
    ...

def johnson(csgraph, directed=..., indices=..., return_predecessors=..., unweighted=...) -> typing.Any:
    "\n    johnson(csgraph, directed=True, indices=None, return_predecessors=False,\n            unweighted=False)\n\n    Compute the shortest path lengths using Johnson's algorithm.\n\n    Johnson's algorithm combines the Bellman-Ford algorithm and Dijkstra's\n    algorithm to quickly find shortest paths in a way that is robust to\n    the presence of negative cycles.  If a negative cycle is detected,\n    an error is raised.  For graphs without negative edge weights,\n    dijkstra may be faster.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array, matrix, or sparse matrix, 2 dimensions\n        The N x N array of distances representing the input graph.\n    directed : bool, optional\n        If True (default), then find the shortest path on a directed graph:\n        only move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i]\n    indices : array_like or int, optional\n        if specified, only compute the paths from the points at the given\n        indices.\n    return_predecessors : bool, optional\n        If True, return the size (N, N) predecesor matrix\n    unweighted : bool, optional\n        If True, then find unweighted distances.  That is, rather than finding\n        the path between each point such that the sum of weights is minimized,\n        find the path such that the number of edges is minimized.\n\n    Returns\n    -------\n    dist_matrix : ndarray\n        The N x N matrix of distances between graph nodes. dist_matrix[i,j]\n        gives the shortest distance from point i to point j along the graph.\n\n    predecessors : ndarray\n        Returned only if return_predecessors == True.\n        The N x N matrix of predecessors, which can be used to reconstruct\n        the shortest paths.  Row i of the predecessor matrix contains\n        information on the shortest paths from point i: each entry\n        predecessors[i, j] gives the index of the previous node in the\n        path from point i to point j.  If no path exists between point\n        i and j, then predecessors[i, j] = -9999\n\n    Raises\n    ------\n    NegativeCycleError:\n        if there are negative cycles in the graph\n\n    Notes\n    -----\n    This routine is specially designed for graphs with negative edge weights.\n    If all edge weights are positive, then Dijkstra's algorithm is a better\n    choice.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import johnson\n\n    >>> graph = [\n    ... [0, 1, 2, 0],\n    ... [0, 0, 0, 1],\n    ... [2, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)\t1\n      (0, 2)\t2\n      (1, 3)\t1\n      (2, 0)\t2\n      (2, 3)\t3\n\n    >>> dist_matrix, predecessors = johnson(csgraph=graph, directed=False, indices=0, return_predecessors=True)\n    >>> dist_matrix\n    array([ 0.,  1.,  2.,  2.])\n    >>> predecessors\n    array([-9999,     0,     0,     1], dtype=int32)\n\n    "
    ...

def shortest_path(csgraph, method=..., directed=..., return_predecessors=..., unweighted=..., overwrite=..., indices=...) -> typing.Any:
    "\n    shortest_path(csgraph, method='auto', directed=True, return_predecessors=False,\n                  unweighted=False, overwrite=False, indices=None)\n\n    Perform a shortest-path graph search on a positive directed or\n    undirected graph.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array, matrix, or sparse matrix, 2 dimensions\n        The N x N array of distances representing the input graph.\n    method : string ['auto'|'FW'|'D'], optional\n        Algorithm to use for shortest paths.  Options are:\n\n           'auto' -- (default) select the best among 'FW', 'D', 'BF', or 'J'\n                     based on the input data.\n\n           'FW'   -- Floyd-Warshall algorithm.  Computational cost is\n                     approximately ``O[N^3]``.  The input csgraph will be\n                     converted to a dense representation.\n\n           'D'    -- Dijkstra's algorithm with Fibonacci heaps.  Computational\n                     cost is approximately ``O[N(N*k + N*log(N))]``, where\n                     ``k`` is the average number of connected edges per node.\n                     The input csgraph will be converted to a csr\n                     representation.\n\n           'BF'   -- Bellman-Ford algorithm.  This algorithm can be used when\n                     weights are negative.  If a negative cycle is encountered,\n                     an error will be raised.  Computational cost is\n                     approximately ``O[N(N^2 k)]``, where ``k`` is the average\n                     number of connected edges per node. The input csgraph will\n                     be converted to a csr representation.\n\n           'J'    -- Johnson's algorithm.  Like the Bellman-Ford algorithm,\n                     Johnson's algorithm is designed for use when the weights\n                     are negative.  It combines the Bellman-Ford algorithm\n                     with Dijkstra's algorithm for faster computation.\n\n    directed : bool, optional\n        If True (default), then find the shortest path on a directed graph:\n        only move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i]\n    return_predecessors : bool, optional\n        If True, return the size (N, N) predecesor matrix\n    unweighted : bool, optional\n        If True, then find unweighted distances.  That is, rather than finding\n        the path between each point such that the sum of weights is minimized,\n        find the path such that the number of edges is minimized.\n    overwrite : bool, optional\n        If True, overwrite csgraph with the result.  This applies only if\n        method == 'FW' and csgraph is a dense, c-ordered array with\n        dtype=float64.\n    indices : array_like or int, optional\n        If specified, only compute the paths from the points at the given\n        indices. Incompatible with method == 'FW'.\n\n    Returns\n    -------\n    dist_matrix : ndarray\n        The N x N matrix of distances between graph nodes. dist_matrix[i,j]\n        gives the shortest distance from point i to point j along the graph.\n    predecessors : ndarray\n        Returned only if return_predecessors == True.\n        The N x N matrix of predecessors, which can be used to reconstruct\n        the shortest paths.  Row i of the predecessor matrix contains\n        information on the shortest paths from point i: each entry\n        predecessors[i, j] gives the index of the previous node in the\n        path from point i to point j.  If no path exists between point\n        i and j, then predecessors[i, j] = -9999\n\n    Raises\n    ------\n    NegativeCycleError:\n        if there are negative cycles in the graph\n\n    Notes\n    -----\n    As currently implemented, Dijkstra's algorithm and Johnson's algorithm\n    do not work for graphs with direction-dependent distances when\n    directed == False.  i.e., if csgraph[i,j] and csgraph[j,i] are non-equal\n    edges, method='D' may yield an incorrect result.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import shortest_path\n\n    >>> graph = [\n    ... [0, 1, 2, 0],\n    ... [0, 0, 0, 1],\n    ... [2, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)\t1\n      (0, 2)\t2\n      (1, 3)\t1\n      (2, 0)\t2\n      (2, 3)\t3\n\n    >>> dist_matrix, predecessors = shortest_path(csgraph=graph, directed=False, indices=0, return_predecessors=True)\n    >>> dist_matrix\n    array([ 0.,  1.,  2.,  2.])\n    >>> predecessors\n    array([-9999,     0,     0,     1], dtype=int32)\n\n    "
    ...

def validate_graph(csgraph, directed, dtype, csr_output, dense_output, copy_if_dense, copy_if_sparse, null_value_in, null_value_out, infinity_null, nan_null) -> typing.Any:
    'Routine for validation and conversion of csgraph inputs'
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Any, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from scipy.sparse.linalg import eigsh as eigsh, svds as svds
from abc import ABCMeta, abstractmethod
from numpy import ndarray
from ..utils.extmath import (
    make_nonnegative as make_nonnegative,
    randomized_svd as randomized_svd,
    safe_sparse_dot as safe_sparse_dot,
)
from scipy.linalg import norm as norm
from numbers import Integral as Integral
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from scipy.sparse import dia_matrix as dia_matrix, issparse as issparse
from ..base import BaseEstimator, BiclusterMixin
from .._typing import MatrixLike, Int, ArrayLike
from ..utils import (
    check_random_state as check_random_state,
    check_scalar as check_scalar,
)
from ..utils.validation import assert_all_finite as assert_all_finite
from . import KMeans as KMeans, MiniBatchKMeans as MiniBatchKMeans

BaseSpectral_Self = TypeVar("BaseSpectral_Self", bound="BaseSpectral")


import numpy as np


__all__ = ["SpectralCoclustering", "SpectralBiclustering"]


class BaseSpectral(BiclusterMixin, BaseEstimator, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        n_clusters: int | tuple[int, int] = 3,
        svd_method: str = "randomized",
        n_svd_vecs=None,
        mini_batch: bool = False,
        init: str = "k-means++",
        n_init: int = 10,
        random_state: None | int = None,
    ) -> None:
        ...

    def fit(self: BaseSpectral_Self, X: MatrixLike, y: Any = None) -> BaseSpectral_Self:
        ...


class SpectralCoclustering(BaseSpectral):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    biclusters_: tuple[ndarray, ndarray] = ...
    column_labels_: ArrayLike = ...
    row_labels_: ArrayLike = ...
    columns_: ArrayLike = ...
    rows_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_clusters: Int = 3,
        *,
        svd_method: Literal["randomized", "arpack", "randomized"] = "randomized",
        n_svd_vecs: None | Int = None,
        mini_batch: bool = False,
        init: MatrixLike | Literal["k-means++", "random", "k-means++"] = "k-means++",
        n_init: Int = 10,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...


class SpectralBiclustering(BaseSpectral):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    biclusters_: tuple[ndarray, ndarray] = ...
    column_labels_: ArrayLike = ...
    row_labels_: ArrayLike = ...
    columns_: ArrayLike = ...
    rows_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_clusters: tuple[int, int] | int = 3,
        *,
        method: Literal[
            "bistochastic", "scale", "log", "bistochastic"
        ] = "bistochastic",
        n_components: Int = 6,
        n_best: Int = 3,
        svd_method: Literal["randomized", "arpack", "randomized"] = "randomized",
        n_svd_vecs: None | Int = None,
        mini_batch: bool = False,
        init: MatrixLike | Literal["k-means++", "random", "k-means++"] = "k-means++",
        n_init: Int = 10,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...



"""
This type stub file was generated by pyright.
"""

from IPython.core.magic import Magics, line_magic, magics_class

"""Implementation of configuration-related magic functions.
"""
reg = ...
@magics_class
class ConfigMagics(Magics):
    def __init__(self, shell) -> None:
        ...
    
    @line_magic
    def config(self, s): # -> None:
        """configure IPython

            %config Class[.trait=value]

        This magic exposes most of the IPython config system. Any
        Configurable class should be able to be configured with the simple
        line::

            %config Class.trait=value

        Where `value` will be resolved in the user's namespace, if it is an
        expression or variable name.

        Examples
        --------

        To see what classes are available for config, pass no arguments::

            In [1]: %config
            Available objects for config:
                AliasManager
                DisplayFormatter
                HistoryManager
                IPCompleter
                LoggingMagics
                MagicsManager
                OSMagics
                PrefilterManager
                ScriptMagics
                TerminalInteractiveShell

        To view what is configurable on a given class, just pass the class
        name::

            In [2]: %config LoggingMagics
            LoggingMagics(Magics) options
            ---------------------------
            LoggingMagics.quiet=<Bool>
                Suppress output of log state when logging is enabled
                Current: False

        but the real use is in setting values::

            In [3]: %config LoggingMagics.quiet = True

        and these values are read from the user_ns if they are variables::

            In [4]: feeling_quiet=False

            In [5]: %config LoggingMagics.quiet = feeling_quiet

        """
        ...
    





from functools import reduce

from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = ["immediate_dominators", "dominance_frontiers"]

def immediate_dominators(G: Graph, start): ...
def dominance_frontiers(G: Graph, start): ...



from typing import Mapping
from ...utils import not_implemented_for
from ...classes.graph import Graph

__all__ = [
    "subgraph_centrality_exp",
    "subgraph_centrality",
    "communicability_betweenness_centrality",
    "estrada_index",
]

def subgraph_centrality_exp(G: Graph) -> Mapping: ...
def subgraph_centrality(G: Graph) -> Mapping: ...
def communicability_betweenness_centrality(G: Graph) -> Mapping: ...
def estrada_index(G: Graph) -> float: ...



__version__: str = ...

# These are imported in order as listed

from .exception import (
    HasACycle as HasACycle,
    NodeNotFound as NodeNotFound,
    PowerIterationFailedConvergence as PowerIterationFailedConvergence,
    ExceededMaxIterations as ExceededMaxIterations,
    AmbiguousSolution as AmbiguousSolution,
    NetworkXAlgorithmError as NetworkXAlgorithmError,
    NetworkXException as NetworkXException,
    NetworkXError as NetworkXError,
    NetworkXNoCycle as NetworkXNoCycle,
    NetworkXNoPath as NetworkXNoPath,
    NetworkXNotImplemented as NetworkXNotImplemented,
    NetworkXPointlessConcept as NetworkXPointlessConcept,
    NetworkXUnbounded as NetworkXUnbounded,
    NetworkXUnfeasible as NetworkXUnfeasible,
)

from . import utils as utils

from . import classes as classes
from .classes import filters as filters
from .classes import *

from . import convert as convert
from .convert import (
    to_networkx_graph as to_networkx_graph,
    from_dict_of_dicts as from_dict_of_dicts,
    to_dict_of_dicts as to_dict_of_dicts,
    from_dict_of_lists as from_dict_of_lists,
    to_dict_of_lists as to_dict_of_lists,
    from_edgelist as from_edgelist,
    to_edgelist as to_edgelist,
)

from . import convert_matrix as convert_matrix
from .convert_matrix import (
    from_numpy_matrix as from_numpy_matrix,
    to_numpy_matrix as to_numpy_matrix,
    from_pandas_adjacency as from_pandas_adjacency,
    to_pandas_adjacency as to_pandas_adjacency,
    from_pandas_edgelist as from_pandas_edgelist,
    to_pandas_edgelist as to_pandas_edgelist,
    to_numpy_recarray as to_numpy_recarray,
    from_scipy_sparse_array as from_scipy_sparse_array,
    from_scipy_sparse_matrix as from_scipy_sparse_matrix,
    to_scipy_sparse_array as to_scipy_sparse_array,
    to_scipy_sparse_matrix as to_scipy_sparse_matrix,
    from_numpy_array as from_numpy_array,
    to_numpy_array as to_numpy_array,
)

from . import relabel as relabel
from .relabel import convert_node_labels_to_integers as convert_node_labels_to_integers, relabel_nodes as relabel_nodes

from . import generators as generators
from .generators import *

from . import readwrite as readwrite
from .readwrite import *

# Need to test with SciPy, when available
from . import algorithms as algorithms
from .algorithms import *

from . import linalg as linalg
from .linalg import *

from .testing.test import run as test

from . import drawing as drawing
from .drawing import *



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.integrate._dop, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def dop853(fcn, x, y, xend, rtol, atol, solout, iout, work, iwork, fcn_extra_args=..., overwrite_y=..., solout_extra_args=...) -> typing.Any:
    "x,y,iwork,idid = dop853(fcn,x,y,xend,rtol,atol,solout,iout,work,iwork,[fcn_extra_args,overwrite_y,solout_extra_args])\n\nWrapper for ``dop853``.\n\nParameters\n----------\nfcn : call-back function\nx : input float\ny : input rank-1 array('d') with bounds (n)\nxend : input float\nrtol : input rank-1 array('d') with bounds (*)\natol : input rank-1 array('d') with bounds (*)\nsolout : call-back function\niout : input int\nwork : input rank-1 array('d') with bounds (*)\niwork : input rank-1 array('i') with bounds (*)\n\nOther Parameters\n----------------\nfcn_extra_args : input tuple, optional\n    Default: ()\noverwrite_y : input int, optional\n    Default: 0\nsolout_extra_args : input tuple, optional\n    Default: ()\n\nReturns\n-------\nx : float\ny : rank-1 array('d') with bounds (n)\niwork : rank-1 array('i') with bounds (*)\nidid : int\n\nNotes\n-----\nCall-back functions::\n\n  def fcn(x,y): return f\n  Required arguments:\n    x : input float\n    y : input rank-1 array('d') with bounds (n)\n  Return objects:\n    f : rank-1 array('d') with bounds (n)\n  def solout(nr,xold,x,y,con,icomp,[nd]): return irtn\n  Required arguments:\n    nr : input int\n    xold : input float\n    x : input float\n    y : input rank-1 array('d') with bounds (n)\n    con : input rank-1 array('d') with bounds (5 * nd)\n    icomp : input rank-1 array('i') with bounds (nd)\n  Optional arguments:\n    nd : input int, optional\n    Default: (len(con))/(5)\n  Return objects:\n    irtn : int\n"
    ...

def dopri5(fcn, x, y, xend, rtol, atol, solout, iout, work, iwork, fcn_extra_args=..., overwrite_y=..., solout_extra_args=...) -> typing.Any:
    "x,y,iwork,idid = dopri5(fcn,x,y,xend,rtol,atol,solout,iout,work,iwork,[fcn_extra_args,overwrite_y,solout_extra_args])\n\nWrapper for ``dopri5``.\n\nParameters\n----------\nfcn : call-back function\nx : input float\ny : input rank-1 array('d') with bounds (n)\nxend : input float\nrtol : input rank-1 array('d') with bounds (*)\natol : input rank-1 array('d') with bounds (*)\nsolout : call-back function\niout : input int\nwork : input rank-1 array('d') with bounds (*)\niwork : input rank-1 array('i') with bounds (*)\n\nOther Parameters\n----------------\nfcn_extra_args : input tuple, optional\n    Default: ()\noverwrite_y : input int, optional\n    Default: 0\nsolout_extra_args : input tuple, optional\n    Default: ()\n\nReturns\n-------\nx : float\ny : rank-1 array('d') with bounds (n)\niwork : rank-1 array('i') with bounds (*)\nidid : int\n\nNotes\n-----\nCall-back functions::\n\n  def fcn(x,y): return f\n  Required arguments:\n    x : input float\n    y : input rank-1 array('d') with bounds (n)\n  Return objects:\n    f : rank-1 array('d') with bounds (n)\n  def solout(nr,xold,x,y,con,icomp,[nd]): return irtn\n  Required arguments:\n    nr : input int\n    xold : input float\n    x : input float\n    y : input rank-1 array('d') with bounds (n)\n    con : input rank-1 array('d') with bounds (5 * nd)\n    icomp : input rank-1 array('i') with bounds (nd)\n  Optional arguments:\n    nd : input int, optional\n    Default: (len(con))/(5)\n  Return objects:\n    irtn : int\n"
    ...

def types() -> typing.Any:
    "'i'-scalar\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.sparse._csparsetools, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def _lil_fancy_get_int32() -> typing.Any:
    ...

def _lil_fancy_get_int64() -> typing.Any:
    ...

def _lil_fancy_set_int32_bool_() -> typing.Any:
    ...

def _lil_fancy_set_int32_clongdouble() -> typing.Any:
    ...

def _lil_fancy_set_int32_complex128() -> typing.Any:
    ...

def _lil_fancy_set_int32_complex64() -> typing.Any:
    ...

def _lil_fancy_set_int32_float32() -> typing.Any:
    ...

def _lil_fancy_set_int32_float64() -> typing.Any:
    ...

def _lil_fancy_set_int32_int16() -> typing.Any:
    ...

def _lil_fancy_set_int32_int32() -> typing.Any:
    ...

def _lil_fancy_set_int32_int64() -> typing.Any:
    ...

def _lil_fancy_set_int32_int8() -> typing.Any:
    ...

def _lil_fancy_set_int32_longdouble() -> typing.Any:
    ...

def _lil_fancy_set_int32_uint16() -> typing.Any:
    ...

def _lil_fancy_set_int32_uint32() -> typing.Any:
    ...

def _lil_fancy_set_int32_uint64() -> typing.Any:
    ...

def _lil_fancy_set_int32_uint8() -> typing.Any:
    ...

def _lil_fancy_set_int64_bool_() -> typing.Any:
    ...

def _lil_fancy_set_int64_clongdouble() -> typing.Any:
    ...

def _lil_fancy_set_int64_complex128() -> typing.Any:
    ...

def _lil_fancy_set_int64_complex64() -> typing.Any:
    ...

def _lil_fancy_set_int64_float32() -> typing.Any:
    ...

def _lil_fancy_set_int64_float64() -> typing.Any:
    ...

def _lil_fancy_set_int64_int16() -> typing.Any:
    ...

def _lil_fancy_set_int64_int32() -> typing.Any:
    ...

def _lil_fancy_set_int64_int64() -> typing.Any:
    ...

def _lil_fancy_set_int64_int8() -> typing.Any:
    ...

def _lil_fancy_set_int64_longdouble() -> typing.Any:
    ...

def _lil_fancy_set_int64_uint16() -> typing.Any:
    ...

def _lil_fancy_set_int64_uint32() -> typing.Any:
    ...

def _lil_fancy_set_int64_uint64() -> typing.Any:
    ...

def _lil_fancy_set_int64_uint8() -> typing.Any:
    ...

def _lil_flatten_to_array_bool_() -> typing.Any:
    ...

def _lil_flatten_to_array_clongdouble() -> typing.Any:
    ...

def _lil_flatten_to_array_complex128() -> typing.Any:
    ...

def _lil_flatten_to_array_complex64() -> typing.Any:
    ...

def _lil_flatten_to_array_float32() -> typing.Any:
    ...

def _lil_flatten_to_array_float64() -> typing.Any:
    ...

def _lil_flatten_to_array_int16() -> typing.Any:
    ...

def _lil_flatten_to_array_int32() -> typing.Any:
    ...

def _lil_flatten_to_array_int64() -> typing.Any:
    ...

def _lil_flatten_to_array_int8() -> typing.Any:
    ...

def _lil_flatten_to_array_longdouble() -> typing.Any:
    ...

def _lil_flatten_to_array_uint16() -> typing.Any:
    ...

def _lil_flatten_to_array_uint32() -> typing.Any:
    ...

def _lil_flatten_to_array_uint64() -> typing.Any:
    ...

def _lil_flatten_to_array_uint8() -> typing.Any:
    ...

def _lil_get_lengths_int32() -> typing.Any:
    ...

def _lil_get_lengths_int64() -> typing.Any:
    ...

def lil_fancy_get() -> typing.Any:
    '\n    Get multiple items at given indices in LIL matrix and store to\n    another LIL.\n\n    Parameters\n    ----------\n    M, N, rows, data\n        LIL matrix data, initially empty\n    new_rows, new_idx\n        Data for LIL matrix to insert to.\n        Must be preallocated to shape `i_idx.shape`!\n    i_idx, j_idx\n        Indices of elements to insert to the new LIL matrix.\n\n    '
    ...

def lil_fancy_set() -> typing.Any:
    '\n    Set multiple items to a LIL matrix.\n\n    Checks for zero elements and deletes them.\n\n    Parameters\n    ----------\n    M, N, rows, data\n        LIL matrix data\n    i_idx, j_idx\n        Indices of elements to insert to the new LIL matrix.\n    values\n        Values of items to set.\n\n    '
    ...

def lil_flatten_to_array() -> typing.Any:
    ...

def lil_get1() -> typing.Any:
    "\n    Get a single item from LIL matrix.\n\n    Doesn't do output type conversion. Checks for bounds errors.\n\n    Parameters\n    ----------\n    M, N, rows, datas\n        Shape and data arrays for a LIL matrix\n    i, j : int\n        Indices at which to get\n\n    Returns\n    -------\n    x\n        Value at indices.\n\n    "
    ...

def lil_get_lengths() -> typing.Any:
    ...

def lil_get_row_ranges() -> typing.Any:
    '\n    Column-slicing fast path for LIL matrices.\n    Extracts values from rows/datas and inserts in to\n    new_rows/new_datas.\n    Parameters\n    ----------\n    M, N\n         Shape of input array\n    rows, datas\n         LIL data for input array, shape (M, N)\n    new_rows, new_datas\n         LIL data for output array, shape (len(irows), nj)\n    irows : iterator\n         Iterator yielding row indices\n    j_start, j_stop, j_stride\n         Column range(j_start, j_stop, j_stride) to get\n    nj : int\n         Number of columns corresponding to j_* variables.\n    '
    ...

def lil_insert() -> typing.Any:
    '\n    Insert a single item to LIL matrix.\n\n    Checks for bounds errors and deletes item if x is zero.\n\n    Parameters\n    ----------\n    M, N, rows, datas\n        Shape and data arrays for a LIL matrix\n    i, j : int\n        Indices at which to get\n    x\n        Value to insert.\n\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Literal, Mapping, Sequence
from numpy.random import RandomState
from scipy import sparse as sparse
from itertools import chain as chain
from ...base import BaseEstimator
from ...utils import (
    check_array as check_array,
    check_matplotlib_support as check_matplotlib_support,
    check_random_state as check_random_state,
)
from ...utils._bunch import Bunch
from .. import partial_dependence as partial_dependence
from ...utils.parallel import delayed as delayed, Parallel as Parallel
from matplotlib.gridspec import GridSpecFromSubplotSpec as GridSpecFromSubplotSpec
from matplotlib.axes import Axes
from numpy import ndarray
from ...base import is_regressor as is_regressor
from matplotlib.figure import Figure
from math import ceil as ceil
from scipy.stats.mstats import mquantiles as mquantiles
from ..._typing import Int, ArrayLike, MatrixLike
import numbers
import warnings

import numpy as np


class PartialDependenceDisplay:
    figure_: Figure = ...
    heatmaps_: ndarray = ...
    bars_: ndarray = ...
    contours_: ndarray = ...
    deciles_hlines_: ndarray = ...
    deciles_vlines_: ndarray = ...
    lines_: ndarray = ...
    axes_: ndarray = ...
    bounding_ax_: None | Axes = ...

    def __init__(
        self,
        pd_results: list[Bunch] | Sequence[Bunch],
        *,
        features: Sequence[int] | Sequence[tuple[int, int]],
        feature_names: Sequence[str],
        target_idx: Int,
        deciles: dict,
        pdp_lim: None | Mapping | str = "deprecated",
        kind: Sequence[Literal["average", "individual", "both"]]
        | Literal["average", "individual", "both", "average"] = "average",
        subsample: float | None | int = 1000,
        random_state: RandomState | None | Int = None,
        is_categorical: None | ArrayLike = None,
    ) -> None:
        ...

    @classmethod
    def from_estimator(
        cls,
        estimator: BaseEstimator,
        X: MatrixLike,
        features: list[int | str | tuple[str, str]]
        | Sequence[int | str | Sequence[int | str]],
        *,
        categorical_features: None | MatrixLike | ArrayLike = None,
        feature_names: None | ArrayLike = None,
        target: None | Int = None,
        response_method: Literal[
            "auto", "predict_proba", "decision_function", "auto"
        ] = "auto",
        n_cols: Int = 3,
        grid_resolution: Int = 100,
        percentiles: tuple[float, ...] = ...,
        method: str = "auto",
        n_jobs: None | Int = None,
        verbose: Int = 0,
        line_kw: None | dict = None,
        ice_lines_kw: None | dict = None,
        pd_line_kw: None | dict = None,
        contour_kw: None | dict = None,
        ax: None | Sequence[Axes] | Axes = None,
        kind: Literal["average", "individual", "both", "average"] = "average",
        centered: bool = False,
        subsample: float | None | int = 1000,
        random_state: RandomState | None | Int = None,
    ) -> PartialDependenceDisplay:
        ...

    def plot(
        self,
        *,
        ax: None | Sequence[Axes] | Axes = None,
        n_cols: Int = 3,
        line_kw: None | dict = None,
        ice_lines_kw: None | dict = None,
        pd_line_kw: None | dict = None,
        contour_kw: None | dict = None,
        bar_kw: None | dict = None,
        heatmap_kw: None | dict = None,
        pdp_lim: None | dict = None,
        centered: bool = False,
    ) -> PartialDependenceDisplay:
        ...



from numpy.typing import ArrayLike
from itertools import chain, combinations, permutations, product

from ...classes.graph import Graph
from ...exception import NetworkXException
from ...utils import arbitrary_element

__all__ = [
    "contracted_edge",
    "contracted_nodes",
    "equivalence_classes",
    "identified_nodes",
    "quotient_graph",
]

chaini = ...

def equivalence_classes(iterable: ArrayLike | tuple | set, relation): ...
def quotient_graph(
    G: Graph,
    partition,
    edge_relation=None,
    node_data=None,
    edge_data=None,
    relabel: bool = False,
    create_using=None,
): ...
def contracted_nodes(G: Graph, u, v, self_loops: bool = True, copy: bool = True): ...

identified_nodes = ...

def contracted_edge(G: Graph, edge: tuple, self_loops: bool = True, copy=True): ...



from typing import Any, Optional, Sequence

from django.apps.config import AppConfig

E001: Any

def check_async_unsafe(
    app_configs: Optional[Sequence[AppConfig]] = ..., **kwargs: Any
) -> Any: ...



from typing import Callable, ClassVar, TypeVar
from ..utils.sparsefuncs import csc_median_axis_0 as csc_median_axis_0
from ..utils.validation import check_is_fitted as check_is_fitted
from scipy import sparse as sp
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Real as Real
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from ..base import BaseEstimator, ClassifierMixin
from ..metrics.pairwise import pairwise_distances_argmin as pairwise_distances_argmin
from .._typing import Float, MatrixLike, ArrayLike
from ..preprocessing import LabelEncoder as LabelEncoder

NearestCentroid_Self = TypeVar("NearestCentroid_Self", bound="NearestCentroid")


# Author: Robert Layton <robertlayton@gmail.com>
#         Olivier Grisel <olivier.grisel@ensta.org>
#
# License: BSD 3 clause

import warnings
import numpy as np


class NearestCentroid(ClassifierMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ndarray = ...
    centroids_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        metric: str | Callable = "euclidean",
        *,
        shrink_threshold: None | Float = None
    ) -> None:
        ...

    def fit(
        self: NearestCentroid_Self, X: MatrixLike | ArrayLike, y: ArrayLike
    ) -> NearestCentroid_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...



from typing import Callable, ClassVar, Literal, TypeVar
from ._base import NeighborsBase, KNeighborsMixin, RadiusNeighborsMixin
from numpy import ndarray
from ..utils._param_validation import StrOptions as StrOptions
from ..base import RegressorMixin
from .._typing import Int, MatrixLike, ArrayLike, Float

KNeighborsRegressor_Self = TypeVar(
    "KNeighborsRegressor_Self", bound="KNeighborsRegressor"
)
RadiusNeighborsRegressor_Self = TypeVar(
    "RadiusNeighborsRegressor_Self", bound="RadiusNeighborsRegressor"
)


# Authors: Jake Vanderplas <vanderplas@astro.washington.edu>
#          Fabian Pedregosa <fabian.pedregosa@inria.fr>
#          Alexandre Gramfort <alexandre.gramfort@inria.fr>
#          Sparseness support by Lars Buitinck
#          Multi-output support by Arnaud Joly <a.joly@ulg.ac.be>
#          Empty radius support by Andreas Bjerre-Nielsen
#
# License: BSD 3 clause (C) INRIA, University of Amsterdam,
#                           University of Copenhagen

import warnings

import numpy as np


class KNeighborsRegressor(KNeighborsMixin, RegressorMixin, NeighborsBase):
    n_samples_fit_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    effective_metric_params_: dict = ...
    effective_metric_: str | Callable = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_neighbors: Int = 5,
        *,
        weights: None
        | Literal["uniform", "distance", "uniform"]
        | Callable = "uniform",
        algorithm: Literal["auto", "ball_tree", "kd_tree", "brute", "auto"] = "auto",
        leaf_size: Int = 30,
        p: Int = 2,
        metric: str | Callable = "minkowski",
        metric_params: None | dict = None,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: KNeighborsRegressor_Self, X: MatrixLike, y: MatrixLike | ArrayLike
    ) -> KNeighborsRegressor_Self:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...


class RadiusNeighborsRegressor(RadiusNeighborsMixin, RegressorMixin, NeighborsBase):
    n_samples_fit_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    effective_metric_params_: dict = ...
    effective_metric_: str | Callable = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        radius: Float = 1.0,
        *,
        weights: None
        | Literal["uniform", "distance", "uniform"]
        | Callable = "uniform",
        algorithm: Literal["auto", "ball_tree", "kd_tree", "brute", "auto"] = "auto",
        leaf_size: Int = 30,
        p: Int = 2,
        metric: str | Callable = "minkowski",
        metric_params: None | dict = None,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: RadiusNeighborsRegressor_Self, X: MatrixLike, y: MatrixLike | ArrayLike
    ) -> RadiusNeighborsRegressor_Self:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...



from typing import Any, Callable, ClassVar, Iterator, Literal, TypeVar
from ._kmeans import _BaseKMeans
from numpy.random import RandomState
from numpy import ndarray
from ..utils.extmath import row_norms as row_norms
from ..utils._param_validation import StrOptions as StrOptions
from .._typing import Float, Int, MatrixLike, ArrayLike
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    check_random_state as check_random_state,
)

BisectingKMeans_Self = TypeVar("BisectingKMeans_Self", bound="BisectingKMeans")

# Author: Michal Krawczyk <mkrwczyk.1@gmail.com>

import warnings

import numpy as np
import scipy.sparse as sp


class _BisectingTree:
    def __init__(self, center: ndarray, indices: ndarray, score: Float) -> None:
        ...

    def split(self, labels: ndarray, centers: ndarray, scores: ndarray) -> None:
        ...

    def get_cluster_to_bisect(self) -> _BisectingTree:
        ...

    def iter_leaves(self) -> Iterator[_BisectingTree]:
        ...


class BisectingKMeans(_BaseKMeans):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    inertia_: float = ...
    labels_: ndarray = ...
    cluster_centers_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_clusters: Int = 8,
        *,
        init: Literal["k-means++", "random", "random"] | Callable = "random",
        n_init: Int = 1,
        random_state: RandomState | None | Int = None,
        max_iter: Int = 300,
        verbose: Int = 0,
        tol: Float = 1e-4,
        copy_x: bool = True,
        algorithm: Literal["lloyd", "elkan", "lloyd"] = "lloyd",
        bisecting_strategy: Literal[
            "biggest_inertia", "largest_cluster", "biggest_inertia"
        ] = "biggest_inertia",
    ) -> None:
        ...

    def fit(
        self: BisectingKMeans_Self,
        X: MatrixLike | ArrayLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> BisectingKMeans_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...



from typing import TypeVar
from abc import ABCMeta, abstractmethod
from scipy import linalg as linalg
from numpy import ndarray
from ..base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from .._typing import MatrixLike
from ..utils.validation import check_is_fitted as check_is_fitted

_BasePCA_Self = TypeVar("_BasePCA_Self", bound="_BasePCA")


# Author: Alexandre Gramfort <alexandre.gramfort@inria.fr>
#         Olivier Grisel <olivier.grisel@ensta.org>
#         Mathieu Blondel <mathieu@mblondel.org>
#         Denis A. Engemann <denis-alexander.engemann@inria.fr>
#         Kyle Kastner <kastnerkyle@gmail.com>
#
# License: BSD 3 clause

import numpy as np


class _BasePCA(
    ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator, metaclass=ABCMeta
):
    def get_covariance(self) -> ndarray:
        ...

    def get_precision(self) -> ndarray:
        ...

    @abstractmethod
    def fit(self: _BasePCA_Self, X: MatrixLike, y=None) -> _BasePCA_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...

    def inverse_transform(self, X: MatrixLike) -> ndarray:
        ...



"""
This type stub file was generated by pyright.
"""

""" Import Qt in a manner suitable for an IPython kernel.

This is the import used for the `gui=qt` or `matplotlib=qt` initialization.

Import Priority:

if Qt has been imported anywhere else:
   use that

if matplotlib has been imported and doesn't support v2 (<= 1.0.1):
    use PyQt4 @v1

Next, ask QT_API env variable

if QT_API not set:
    ask matplotlib what it's using. If Qt4Agg or Qt5Agg, then use the
        version matplotlib is configured with

    else: (matplotlib said nothing)
        # this is the default path - nobody told us anything
        try in this order:
            PyQt default version, PySide, PyQt5
else:
    use what QT_API says

    Note that %gui's implementation will always set a `QT_API`, see
    `IPython.terminal.pt_inputhooks.get_inputhook_name_and_func`

"""
_qt_apis = ...
def matplotlib_options(mpl): # -> None:
    """Constraints placed on an imported matplotlib."""
    ...

def get_options():
    """Return a list of acceptable QT APIs, in decreasing order of preference."""
    ...

api_opts = ...
enum_helper = ...



from typing import Any, ClassVar, Literal, Sequence, TypeVar
from numpy.random import RandomState
from abc import ABCMeta, abstractmethod
from scipy import linalg as linalg
from .base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from .utils.validation import (
    check_array as check_array,
    check_is_fitted as check_is_fitted,
)
from numpy import ndarray
from numbers import Integral as Integral, Real as Real
from .utils.extmath import safe_sparse_dot as safe_sparse_dot
from .exceptions import DataDimensionalityWarning as DataDimensionalityWarning
from .utils import check_random_state as check_random_state
from .utils._param_validation import Interval as Interval, StrOptions as StrOptions
from .utils.random import sample_without_replacement as sample_without_replacement
from scipy.sparse import spmatrix
from ._typing import Float, ArrayLike, Int, MatrixLike

BaseRandomProjection_Self = TypeVar(
    "BaseRandomProjection_Self", bound="BaseRandomProjection"
)

# Authors: Olivier Grisel <olivier.grisel@ensta.org>,
#          Arnaud Joly <a.joly@ulg.ac.be>
# License: BSD 3 clause

import warnings

import numpy as np
import scipy.sparse as sp

__all__ = [
    "SparseRandomProjection",
    "GaussianRandomProjection",
    "johnson_lindenstrauss_min_dim",
]


def johnson_lindenstrauss_min_dim(
    n_samples: Sequence[int] | ndarray | Float, *, eps: float | ArrayLike = 0.1
) -> ndarray | int:
    ...


class BaseRandomProjection(
    TransformerMixin, BaseEstimator, ClassNamePrefixFeaturesOutMixin, metaclass=ABCMeta
):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        n_components: str | Int = "auto",
        *,
        eps: float = 0.1,
        compute_inverse_components: bool = False,
        random_state=None,
    ) -> None:
        ...

    def fit(
        self: BaseRandomProjection_Self, X: MatrixLike, y: Any = None
    ) -> BaseRandomProjection_Self | SparseRandomProjection:
        ...

    def inverse_transform(self, X: MatrixLike) -> ndarray:
        ...


class GaussianRandomProjection(BaseRandomProjection):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    inverse_components_: ndarray = ...
    components_: ndarray = ...
    n_components_: int = ...

    def __init__(
        self,
        n_components: Literal["auto", "auto"] | Int = "auto",
        *,
        eps: Float = 0.1,
        compute_inverse_components: bool = False,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...


class SparseRandomProjection(BaseRandomProjection):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    density_: float = ...
    inverse_components_: ndarray = ...
    components_: spmatrix = ...
    n_components_: int = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: Literal["auto", "auto"] | Int = "auto",
        *,
        density: float | Literal["auto", "auto"] = "auto",
        eps: Float = 0.1,
        dense_output: bool = False,
        compute_inverse_components: bool = False,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def transform(self, X: MatrixLike) -> ndarray | spmatrix:
        ...



from ctypes import c_char_p
from typing import Any, Optional

class gdal_char_p(c_char_p): ...

def bool_output(func: Any, argtypes: Any, errcheck: Optional[Any] = ...) -> Any: ...
def double_output(
    func: Any, argtypes: Any, errcheck: bool = ..., strarg: bool = ..., cpl: bool = ...
) -> Any: ...
def geom_output(func: Any, argtypes: Any, offset: Optional[Any] = ...) -> Any: ...
def int_output(func: Any, argtypes: Any, errcheck: Optional[Any] = ...) -> Any: ...
def int64_output(func: Any, argtypes: Any) -> Any: ...
def srs_output(func: Any, argtypes: Any) -> Any: ...
def const_string_output(
    func: Any,
    argtypes: Any,
    offset: Optional[Any] = ...,
    decoding: Optional[Any] = ...,
    cpl: bool = ...,
) -> Any: ...
def string_output(
    func: Any,
    argtypes: Any,
    offset: int = ...,
    str_result: bool = ...,
    decoding: Optional[Any] = ...,
) -> Any: ...
def void_output(
    func: Any, argtypes: Any, errcheck: bool = ..., cpl: bool = ...
) -> Any: ...
def voidptr_output(func: Any, argtypes: Any, errcheck: bool = ...) -> Any: ...
def chararray_output(func: Any, argtypes: Any, errcheck: bool = ...) -> Any: ...



from matplotlib.axes._axes import Axes
from numpy.typing import ArrayLike
from typing import (
    Dict,
    List,
    Optional,
    Tuple,
    Union,
    Literal,
    Mapping,
    Callable,
    Any,
    Sequence,
)
from matplotlib.backend_bases import MouseButton
from matplotlib.figure import Figure

from contextlib import ExitStack
import copy
from numbers import Integral, Number

import numpy as np

import matplotlib as mpl
from . import _api, _docstring, backend_tools, cbook, colors, ticker, transforms
from .lines import Line2D
from .patches import Circle, Rectangle, Ellipse, Polygon
from .transforms import TransformedPatchPath, Affine2D
from matplotlib.lines import Line2D
from matplotlib.patches import Ellipse, Rectangle
from matplotlib.transforms import Affine2D
from numpy import float64, ndarray

class LockDraw:
    def __init__(self) -> None: ...
    def __call__(self, o: "Lasso") -> None: ...
    def release(self, o: "Lasso") -> None: ...
    def available(self, o: "Lasso") -> bool: ...
    def isowner(self, o: "Lasso") -> bool: ...
    def locked(self) -> bool: ...

class Widget:
    drawon: bool = ...
    eventson: bool = ...
    _active: bool = ...

    def set_active(self, active: bool): ...
    def get_active(self) -> bool: ...
    active = ...
    def ignore(self, event: Event) -> bool: ...

class AxesWidget(Widget):
    def __init__(self, ax: Axes) -> None: ...
    def connect_event(self, event: str, callback: Callable) -> None: ...
    def disconnect_events(self) -> None: ...

class Button(AxesWidget):
    def __init__(
        self,
        ax: Axes,
        label: str,
        image: ArrayLike | Image | None = None,
        color: Color = "0.85",
        hovercolor: Color = "0.95",
    ) -> None: ...
    def on_clicked(self, func: Callable) -> int: ...
    def disconnect(self, cid): ...

class SliderBase(AxesWidget):
    def __init__(
        self,
        ax: Axes,
        orientation: Literal["horizontal", "vertical"],
        closedmin: bool,
        closedmax: bool,
        valmin: float,
        valmax: float,
        valfmt: str | None,
        dragging: bool,
        valstep: float | ArrayLike | None,
    ) -> None: ...
    def disconnect(self, cid: int): ...
    def reset(self): ...

class Slider(SliderBase):

    val: float

    def __init__(
        self,
        ax: Axes,
        label: str,
        valmin: float,
        valmax: float,
        valinit: float = 0.5,
        valfmt: str | None = None,
        closedmin: bool = True,
        closedmax: bool = True,
        slidermin: Slider | None = None,
        slidermax: Slider | None = None,
        dragging: bool = True,
        valstep: float | ArrayLike | None = None,
        orientation: Literal["horizontal", "vertical"] = "horizontal",
        *,
        initcolor: Color = "r",
        track_color: Color = "lightgrey",
        handle_style: Mapping | None = None,
        **kwargs,
    ) -> None: ...
    def set_val(self, val: float) -> None: ...
    def on_changed(self, func: Callable) -> int: ...

class RangeSlider(SliderBase):

    val: tuple[float, ...]

    def __init__(
        self,
        ax: Axes,
        label: str,
        valmin: float,
        valmax: float,
        valinit: tuple[float, ...] | None = None,
        valfmt: str | None = None,
        closedmin: bool = True,
        closedmax: bool = True,
        dragging: bool = True,
        valstep: float | None = None,
        orientation: Literal["horizontal", "vertical"] = "horizontal",
        track_color: Color = "lightgrey",
        handle_style: Mapping | None = None,
        **kwargs,
    ) -> None: ...
    def set_min(self, min: float): ...
    def set_max(self, max: float): ...
    def set_val(self, val: Sequence[float]): ...
    def on_changed(self, func: Callable) -> int: ...

class CheckButtons(AxesWidget):
    def __init__(
        self, ax: Axes, labels: Sequence[str], actives: Sequence[bool] = ...
    ) -> None: ...
    def set_active(self, index: int): ...
    def get_status(self) -> tuple[bool, ...]: ...
    def on_clicked(self, func: Callable) -> int: ...
    def disconnect(self, cid) -> None: ...

class TextBox(AxesWidget):

    DIST_FROM_LEFT = ...
    def __init__(
        self,
        ax: Axes,
        label: str,
        initial: str = "",
        color: Color = ".95",
        hovercolor: Color = "1",
        label_pad: float = 0.01,
        textalignment: Literal["left", "center", "right"] = "left",
    ) -> None: ...
    @property
    def text(self) -> str: ...
    def set_val(self, val: str) -> None: ...
    def begin_typing(self, x): ...
    def stop_typing(self) -> None: ...
    def position_cursor(self, x: float): ...
    def on_text_change(self, func: Callable): ...
    def on_submit(self, func: Callable) -> int: ...
    def disconnect(self, cid): ...

class RadioButtons(AxesWidget):

    ax: Axes
    activecolor: Color
    labels: list[Text]
    circles: list[Circle]
    value_selected: str

    def __init__(
        self,
        ax: Axes,
        labels: Sequence[Text],
        active: int = 0,
        activecolor: Color = "blue",
    ) -> None: ...
    def set_active(self, index: int): ...
    def on_clicked(self, func: Callable) -> int: ...
    def disconnect(self, cid): ...

class SubplotTool(Widget):
    def __init__(self, targetfig: Figure, toolfig: Figure) -> None: ...

class Cursor(AxesWidget):
    def __init__(
        self,
        ax: Axes,
        horizOn: bool = True,
        vertOn: bool = True,
        useblit: bool = False,
        **lineprops
    ) -> None: ...
    def clear(self, event: DrawEvent) -> None: ...
    def onmove(self, event: Event): ...

class MultiCursor(Widget):
    def __init__(
        self,
        canvas: FigureCanvasBase,
        axes: Sequence[Axes],
        useblit: bool = True,
        horizOn: bool = False,
        vertOn: bool = True,
        **lineprops
    ) -> None: ...
    def connect(self) -> None: ...
    def disconnect(self): ...
    def clear(self, event: DrawEvent) -> None: ...
    def onmove(self, event: Event): ...

class _SelectorWidget(AxesWidget):
    def __init__(
        self,
        ax: Axes,
        onselect: Callable,
        useblit: bool = False,
        button=None,
        state_modifier_keys=None,
        use_data_coordinates=False,
    ): ...

    eventpress = ...
    eventrelease = ...
    state = ...
    state_modifier_keys = ...
    def set_active(self, active): ...
    def update_background(self, event: DrawEvent) -> None: ...
    def connect_default_events(self) -> None: ...
    def ignore(self, event: Event): ...
    def update(self) -> bool: ...
    def press(self, event: Event): ...
    def release(self, event: Event): ...
    def onmove(self, event: Event): ...
    def on_scroll(self, event: Event): ...
    def on_key_press(self, event: KeyEvent) -> None: ...
    def on_key_release(self, event: Event): ...
    def set_visible(self, visible: bool) -> None: ...
    def clear(self): ...
    @property
    def artists(
        self,
    ) -> tuple[Artist, ...]: ...
    def set_props(self, **props): ...
    def set_handle_props(self, **handle_props): ...
    def add_state(self, state: str): ...
    def remove_state(self, state: str): ...

class SpanSelector(_SelectorWidget):
    def __init__(
        self,
        ax: Axes,
        onselect: Callable,
        direction: Literal["horizontal", "vertical"],
        minspan: float = 0,
        useblit: bool = False,
        props: Mapping | None = None,
        onmove_callback: Callable[[float, float]] | None = None,
        interactive: bool = False,
        button: MouseButton | Sequence[MouseButton] | None = None,
        handle_props: Mapping | None = None,
        grab_range: float = 10,
        state_modifier_keys: Mapping | None = None,
        drag_from_anywhere: bool = False,
        ignore_event_outside: bool = False,
        snap_values: ArrayLike | None = None,
    ): ...

    rect = ...
    rectprops = ...
    active_handle = ...
    pressv = ...
    span_stays = ...
    prev = ...
    def new_axes(self, ax: Axes): ...
    def connect_default_events(self) -> None: ...
    @property
    def direction(self): ...
    @direction.setter
    def direction(self, direction): ...
    @property
    def extents(self): ...
    @extents.setter
    def extents(self, extents): ...

class ToolLineHandles:
    def __init__(
        self,
        ax: Axes,
        positions: ArrayLike,
        direction: Literal["horizontal", "vertical"],
        line_props: Mapping | None = None,
        useblit: bool = True,
    ) -> None: ...
    @property
    def artists(self) -> tuple[Line2D, Line2D]: ...
    @property
    def positions(self): ...
    @property
    def direction(self) -> str: ...
    def set_data(self, positions: tuple): ...
    def set_visible(self, value): ...
    def set_animated(self, value): ...
    def remove(self): ...
    def closest(self, x: float, y: float) -> tuple[int, float]: ...

class ToolHandles:
    def __init__(
        self,
        ax: Axes,
        x: ArrayLike,
        y: ArrayLike,
        marker: str = "o",
        marker_props: Mapping | None = None,
        useblit: bool = True,
    ) -> None: ...
    @property
    def x(self): ...
    @property
    def y(self): ...
    @property
    def artists(self) -> tuple[Line2D]: ...
    def set_data(
        self,
        pts: tuple[float, float, float],
        y: tuple[float, float, float]|None = None,
    ) -> None: ...
    def set_visible(self, val): ...
    def set_animated(self, val): ...
    def closest(self, x: float, y: float) -> tuple[int, float]: ...

class RectangleSelector(_SelectorWidget):
    def __init__(
        self,
        ax: Axes,
        onselect: Callable,
        drawtype="box",
        minspanx: float = 0,
        minspany: float = 0,
        useblit: bool = False,
        lineprops=None,
        props: Mapping | None = None,
        spancoords: Literal["data", "pixels"] = "data",
        button: MouseButton | Sequence[MouseButton] | None = None,
        grab_range: float = 10,
        handle_props: Mapping | None = None,
        interactive: bool = False,
        state_modifier_keys: Mapping | None = None,
        drag_from_anywhere: bool = False,
        ignore_event_outside: bool = False,
        use_data_coordinates: bool = False,
    ): ...

    to_draw = ...
    drawtype = ...
    active_handle = ...
    interactive = ...
    maxdist = ...
    @property
    def corners(self) -> tuple[ndarray, ndarray]: ...
    @property
    def edge_centers(self) -> tuple[ndarray, ndarray]: ...
    @property
    def center(self) -> tuple[float, float]: ...
    @property
    def extents(self): ...
    @extents.setter
    def extents(self, extents): ...
    @property
    def rotation(self) -> float: ...
    @rotation.setter
    def rotation(self, value: float): ...

    draw_shape = ...
    @property
    def geometry(self) -> np.ndarray: ...

class EllipseSelector(RectangleSelector):

    draw_shape = ...

class LassoSelector(_SelectorWidget):
    def __init__(
        self,
        ax: Axes,
        onselect: Callable | None = None,
        useblit: bool = True,
        props: Mapping | None = None,
        button: MouseButton | Sequence[MouseButton] | None = None,
    ): ...
    @_api.deprecated("3.5", alternative="press")
    def onpress(self, event): ...
    def onrelease(self, event): ...

class PolygonSelector(_SelectorWidget):
    def __init__(
        self,
        ax: Axes,
        onselect: Callable,
        useblit: bool = False,
        props: Mapping | None = None,
        handle_props: Mapping | None = None,
        grab_range: float = 10,
        *,
        draw_bounding_box: bool = False,
        box_handle_props: Mapping | None = None,
        box_props: Mapping | None = None,
    ): ...


    line = ...
    vertex_select_radius = ...
    def onmove(self, event: Event): ...
    @property
    def verts(self): ...
    @verts.setter
    def verts(self, xys): ...

class Lasso(AxesWidget):
    def __init__(
        self,
        ax: Axes,
        xy: tuple[float, float],
        callback: Callable | None = None,
        useblit: bool = True,
    ) -> None: ...
    def onrelease(self, event: MouseEvent) -> None: ...
    def onmove(self, event: MouseEvent) -> None: ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

def gaussian_filter(data, sigma): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.stats.statlib, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def gscale(test, other) -> typing.Any:
    "astart,a1,ifault = gscale(test,other)\n\nWrapper for ``gscale``.\n\nParameters\n----------\ntest : input int\nother : input int\n\nReturns\n-------\nastart : float\na1 : rank-1 array('f') with bounds (l1)\nifault : int\n"
    ...

def prho(n, is_) -> typing.Any:
    'ifault = prho(n,is)\n\nWrapper for ``prho``.\n\nParameters\n----------\nn : input int\nis : input int\n\nReturns\n-------\nprho : float\nifault : int\n'
    ...

def swilk(x, a, init=..., n1=...) -> typing.Any:
    "a,w,pw,ifault = swilk(x,a,[init,n1])\n\nWrapper for ``swilk``.\n\nParameters\n----------\nx : input rank-1 array('f') with bounds (n)\na : input rank-1 array('f') with bounds (n2)\n\nOther Parameters\n----------------\ninit : input int, optional\n    Default: 0\nn1 : input int, optional\n    Default: n\n\nReturns\n-------\na : rank-1 array('f') with bounds (n2)\nw : float\npw : float\nifault : int\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Any, ClassVar, TypeVar
from scipy import linalg as linalg, sparse as sparse
from ._base import _BasePCA
from numpy import ndarray
from ..utils._param_validation import Interval as Interval
from ..utils.extmath import svd_flip as svd_flip
from numbers import Integral as Integral
from .._typing import Int, MatrixLike, ArrayLike
from ..utils import gen_batches as gen_batches

IncrementalPCA_Self = TypeVar("IncrementalPCA_Self", bound="IncrementalPCA")

import numpy as np


class IncrementalPCA(_BasePCA):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    batch_size_: int = ...
    n_samples_seen_: int = ...
    n_components_: int = ...
    noise_variance_: float = ...
    var_: ndarray = ...
    mean_: ndarray = ...
    singular_values_: ndarray = ...
    explained_variance_ratio_: ndarray = ...
    explained_variance_: ndarray = ...
    components_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: None | Int = None,
        *,
        whiten: bool = False,
        copy: bool = True,
        batch_size: None | Int = None
    ) -> None:
        ...

    def fit(
        self: IncrementalPCA_Self, X: MatrixLike | ArrayLike, y: Any = None
    ) -> IncrementalPCA_Self:
        ...

    def partial_fit(
        self: IncrementalPCA_Self,
        X: MatrixLike,
        y: Any = None,
        check_input: bool = True,
    ) -> IncrementalPCA_Self:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...



"""
This type stub file was generated by pyright.
"""

"""A fancy version of Python's builtin :func:`dir` function.
"""
def safe_hasattr(obj, attr):
    """In recent versions of Python, hasattr() only catches AttributeError.
    This catches all errors.
    """
    ...

def dir2(obj):
    """dir2(obj) -> list of strings

    Extended version of the Python builtin dir(), which does a few extra
    checks.

    This version is guaranteed to return only a list of true strings, whereas
    dir() returns anything that objects inject into themselves, even if they
    are later not really valid for attribute access (many extension libraries
    have such bugs).
    """
    ...

def get_real_method(obj, name): # -> None:
    """Like getattr, but with a few extra sanity checks:

    - If obj is a class, ignore everything except class methods
    - Check if obj is a proxy that claims to have all attributes
    - Catch attribute access failing with any exception
    - Check that the attribute is a callable object

    Returns the method or None.
    """
    ...




from .pre_build_helpers import compile_test_program as compile_test_program

# This code is adapted for a large part from the astropy openmp helpers, which
# can be found at: https://github.com/astropy/extension-helpers/blob/master/extension_helpers/_openmp_helpers.py  # noqa


import os
import sys
import textwrap
import warnings


def get_openmp_flag(compiler):
    ...


def check_openmp_support():
    ...



import numpy as np
from typing import Any, Callable
from matplotlib._typing import *
from matplotlib.transforms import Affine2DBase, Transform
from matplotlib.text import Text
from matplotlib.font_manager import FontProperties
from matplotlib.figure import Figure

from matplotlib import _api
from matplotlib.backend_bases import (
    FigureCanvasBase,
    FigureManagerBase,
    GraphicsContextBase,
    RendererBase,
    _Backend,
)

class __getattr__:
    NO_ESCAPE = ...
    re_mathsep = ...

def get_fontspec() -> str: ...
def get_preamble() -> str: ...

latex_pt_to_in: float = ...
latex_in_to_pt: float = ...
mpl_pt_to_in: float = ...
mpl_in_to_pt: float = ...

def common_texification(text)-> str: ...
def writeln(fh, line)-> None: ...
def make_pdf_to_png_converter() -> Callable: ...

class LatexError(Exception):
    def __init__(self, message, latex_output=...) -> None: ...
    def __str__(self) -> str: ...

class LatexManager:
    def __init__(self) -> None: ...

    str_cache = ...
    def get_width_height_descent(self, text, prop) -> tuple[float, float, float]: ...

class RendererPgf(RendererBase):
    def __init__(self, figure: Figure, fh: FileLike) -> None: ...
    def draw_markers(
        self,
        gc: GraphicsContextBase,
        marker_path,
        marker_trans: Transform,
        path,
        trans: Transform,
        rgbFace=...,
    )-> None: ...
    def draw_path(self, gc, path, transform, rgbFace=...)-> None: ...
    def option_scale_image(self)-> bool: ...
    def option_image_nocomposite(self)-> bool: ...
    def draw_image(
        self,
        gc: GraphicsContextBase,
        x: Scalar,
        y: Scalar,
        im,
        transform: Affine2DBase = ...,
    )-> None: ...
    def draw_tex(self, gc, x, y, s, prop, angle, ismath=..., mtext=...)-> None: ...
    def draw_text(
        self,
        gc: GraphicsContextBase,
        x: float,
        y: float,
        s: str,
        prop: FontProperties,
        angle: float,
        ismath=...,
        mtext: Text = ...,
    )-> None: ...
    def get_text_width_height_descent(
        self, s: str, prop: FontProperties, ismath: str
    )-> tuple[float, float, float]: ...
    def flipy(self)-> bool: ...
    def get_canvas_width_height(self)-> tuple[float, float]: ...
    def points_to_pixels(self, points: float | ArrayLike): ...

class FigureCanvasPgf(FigureCanvasBase):
    filetypes: dict[str, str] = ...
    def get_default_filetype(self)-> str: ...
    def print_pgf(self, fname_or_fh, **kwargs) -> None: ...
    def print_pdf(self, fname_or_fh, *, metadata=..., **kwargs) -> None: ...
    def print_png(self, fname_or_fh, **kwargs) -> None: ...
    def get_renderer(self) -> RendererPgf: ...
    def draw(self)-> None: ...

FigureManagerPgf = FigureManagerBase

class _BackendPgf(_Backend):
    FigureCanvas = FigureCanvasPgf

class PdfPages:
    def __init__(
        self,
        filename: str | PathLike,
        *,
        keep_empty: bool = True,
        metadata: dict[str, Any] = ...
    ) -> None: ...
    def __enter__(self)-> PdfPages: ...
    def __exit__(self, exc_type, exc_val, exc_tb)-> None: ...
    def close(self) -> None: ...
    def savefig(self, figure: Figure | int = ..., **kwargs) -> None: ...
    def get_pagecount(self) -> int: ...



from ...classes.multidigraph import MultiDiGraph
from ...classes.digraph import DiGraph
from typing import Literal
from ...algorithms.tree.branchings import ArborescenceIterator

# TODO: Implement method from Gabow, Galil, Spence and Tarjan:
#
# @article{
#    year={1986},
#    issn={0209-9683},
#    journal={Combinatorica},
#    volume={6},
#    number={2},
#    doi={10.1007/BF02579168},
#    title={Efficient algorithms for finding minimum spanning trees in
#        undirected and directed graphs},
#    url={https://doi.org/10.1007/BF02579168},
#    publisher={Springer-Verlag},
#    keywords={68 B 15; 68 C 05},
#    author={Gabow, Harold N. and Galil, Zvi and Spencer, Thomas and Tarjan,
#        Robert E.},
#    pages={109-122},
#    language={English}
# }
import string
from dataclasses import dataclass, field
from enum import Enum
from operator import itemgetter
from queue import PriorityQueue

from ...classes.graph import Graph
from ...utils import py_random_state

from .recognition import is_arborescence, is_branching

__all__ = [
    "branching_weight",
    "greedy_branching",
    "maximum_branching",
    "minimum_branching",
    "maximum_spanning_arborescence",
    "minimum_spanning_arborescence",
    "ArborescenceIterator",
    "Edmonds",
]

KINDS: set = ...

STYLES: dict = ...

INF = ...

@py_random_state(1)
def random_string(L=15, seed=None): ...
def branching_weight(
    G: DiGraph, attr: str = "weight", default: float = 1
) -> int | float: ...
@py_random_state(4)
def greedy_branching(
    G: DiGraph, attr: str = "weight", default: float = 1, kind: str = "max", seed=None
): ...

class MultiDiGraph_EdgeKey(MultiDiGraph):
    def __init__(self, incoming_graph_data=None, **attr): ...
    def remove_node(self, n): ...
    def remove_nodes_from(self, nbunch): ...
    def add_edge(self, u_for_edge, v_for_edge, key_for_edge, **attr): ...
    def add_edges_from(self, ebunch_to_add, **attr): ...
    def remove_edge_with_key(self, key): ...
    def remove_edges_from(self, ebunch): ...

def get_path(G: Graph, u, v): ...

class Edmonds:
    def __init__(self, G: Graph, seed=None): ...
    def _init(self, attr, default, kind, style, preserve_attrs, seed, partition): ...
    def find_optimum(
        self,
        attr: str = "weight",
        default: float = 1,
        kind: Literal["min", "max"] = "max",
        style: Literal["branching", "arborescence"] = "branching",
        preserve_attrs: bool = False,
        partition: str | None = None,
        seed=None,
    ): ...

def maximum_branching(
    G: Graph,
    attr: str = "weight",
    default: float = 1,
    preserve_attrs: bool = False,
    partition: str | None = None,
): ...
def minimum_branching(
    G: Graph,
    attr: str = "weight",
    default: float = 1,
    preserve_attrs: bool = False,
    partition: str | None = None,
): ...
def maximum_spanning_arborescence(
    G: Graph,
    attr: str = "weight",
    default: float = 1,
    preserve_attrs: bool = False,
    partition: str | None = None,
): ...
def minimum_spanning_arborescence(
    G: Graph,
    attr: str = "weight",
    default: float = 1,
    preserve_attrs: bool = False,
    partition: str | None = None,
): ...

docstring_branching: str = ...

docstring_arborescence = ...

maximum_branching.__doc__ = ...

minimum_branching.__doc__ = ...

maximum_spanning_arborescence.__doc__ = ...

minimum_spanning_arborescence.__doc__ = ...

class ArborescenceIterator:
    ...

    def __init__(
        self,
        G: Graph,
        weight: str = "weight",
        minimum: bool = True,
        init_partition: tuple | None = None,
    ): ...
    def __iter__(self) -> ArborescenceIterator: ...
    def __next__(self): ...
    def _partition(self, partition, partition_arborescence): ...
    def _write_partition(self, partition): ...
    def _clear_partition(self, G): ...



from vispy.gloo.texture import Texture2D
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-

import numpy as np

from ...gloo import (
    Program,
    FrameBuffer,
    VertexBuffer,
    Texture2D,
    set_viewport,
    set_state,
)

vert_seed: str = ...

vert: str = ...

frag_seed: str = ...

frag_flood: str = ...

frag_insert: str = ...

class SDFRendererGPU(object):
    def __init__(self): ...
    def render_to_texture(
        self,
        data: ArrayLike,
        texture: Texture2D,
        offset: tuple[int, ...],
        size: tuple[int, ...],
    ): ...
    def _render_edf(self, orig_tex): ...



"""
This type stub file was generated by pyright.
"""

"""Posix-specific implementation of process utilities.

This file is only meant to be imported by process.py, not by end-users.
"""
class ProcessHandler:
    """Execute subprocesses under the control of pexpect.
    """
    read_timeout = ...
    terminate_timeout = ...
    logfile = ...
    _sh = ...
    @property
    def sh(self):
        ...
    
    def __init__(self, logfile=..., read_timeout=..., terminate_timeout=...) -> None:
        """Arguments are used for pexpect calls."""
        ...
    
    def getoutput(self, cmd): # -> None:
        """Run a command and return its stdout/stderr as a string.

        Parameters
        ----------
        cmd : str
            A command to be executed in the system shell.

        Returns
        -------
        output : str
            A string containing the combination of stdout and stderr from the
        subprocess, in whatever order the subprocess originally wrote to its
        file descriptors (so the order of the information in this string is the
        correct order as would be seen if running the command in a terminal).
        """
        ...
    
    def getoutput_pexpect(self, cmd): # -> None:
        """Run a command and return its stdout/stderr as a string.

        Parameters
        ----------
        cmd : str
            A command to be executed in the system shell.

        Returns
        -------
        output : str
            A string containing the combination of stdout and stderr from the
        subprocess, in whatever order the subprocess originally wrote to its
        file descriptors (so the order of the information in this string is the
        correct order as would be seen if running the command in a terminal).
        """
        ...
    
    def system(self, cmd):
        """Execute a command in a subshell.

        Parameters
        ----------
        cmd : str
            A command to be executed in the system shell.

        Returns
        -------
        int : child's exitstatus
        """
        ...
    


system = ...
def check_pid(pid):
    ...




# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.optimize.minpack2, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def dcsrch(stp, f, g, ftol, gtol, xtol, task, stpmin, stpmax, isave, dsave) -> typing.Any:
    "stp,f,g,task = dcsrch(stp,f,g,ftol,gtol,xtol,task,stpmin,stpmax,isave,dsave)\n\nWrapper for ``dcsrch``.\n\nParameters\n----------\nstp : input float\nf : input float\ng : input float\nftol : input float\ngtol : input float\nxtol : input float\ntask : input string(len=60)\nstpmin : input float\nstpmax : input float\nisave : in/output rank-1 array('i') with bounds (2)\ndsave : in/output rank-1 array('d') with bounds (13)\n\nReturns\n-------\nstp : float\nf : float\ng : float\ntask : string(len=60)\n"
    ...

def dcstep(stx, fx, dx, sty, fy, dy, stp, fp, dp, brackt, stpmin, stpmax) -> typing.Any:
    'stx,fx,dx,sty,fy,dy,stp,brackt = dcstep(stx,fx,dx,sty,fy,dy,stp,fp,dp,brackt,stpmin,stpmax)\n\nWrapper for ``dcstep``.\n\nParameters\n----------\nstx : input float\nfx : input float\ndx : input float\nsty : input float\nfy : input float\ndy : input float\nstp : input float\nfp : input float\ndp : input float\nbrackt : input int\nstpmin : input float\nstpmax : input float\n\nReturns\n-------\nstx : float\nfx : float\ndx : float\nsty : float\nfy : float\ndy : float\nstp : float\nbrackt : int\n'
    ...

def __getattr__(name) -> typing.Any:
    ...




from os import PathLike
import numpy as np
import shutil
from packaging import version

from ._registry import registry, legacy_registry, registry_urls

from .. import __version__

import os.path as osp
import os

legacy_data_dir = ...
skimage_distribution_dir = ...

def _has_hash(path, expected_hash): ...
def create_image_fetcher(): ...

image_fetcher, data_dir = ...

def _skip_pytest_case_requiring_pooch(data_filename): ...
def _fetch(data_filename): ...
def _init_pooch(): ...
def download_all(directory: PathLike | None = None): ...
def lbp_frontal_face_cascade_filename(): ...
def _load(f, as_gray=False): ...
def camera(): ...
def eagle(): ...
def astronaut(): ...
def brick(): ...
def grass(): ...
def gravel(): ...
def text(): ...
def checkerboard(): ...
def cells3d(): ...
def human_mitosis(): ...
def cell(): ...
def coins(): ...
def kidney(): ...
def lily(): ...
def logo(): ...
def microaneurysms(): ...
def moon(): ...
def page(): ...
def horse(): ...
def clock(): ...
def immunohistochemistry(): ...
def chelsea(): ...

# Define an alias for chelsea that is more descriptive.
cat = ...

def coffee(): ...
def hubble_deep_field(): ...
def retina(): ...
def shepp_logan_phantom(): ...
def colorwheel(): ...
def rocket(): ...
def stereo_motorcycle(): ...
def lfw_subset(): ...
def skin(): ...
def nickel_solidification(): ...
def protein_transport(): ...
def brain(): ...
def vortex(): ...



from typing import Any, Callable, ClassVar, Literal, Mapping, TypeVar
from numpy.random import RandomState
from ..linear_model._base import LinearClassifierMixin, SparseCoefMixin, LinearModel
from ._base import BaseSVC, BaseLibSVM
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from ..base import BaseEstimator, RegressorMixin, OutlierMixin
from .._typing import Float, Int, MatrixLike, ArrayLike
from ..utils import deprecated

OneClassSVM_Self = TypeVar("OneClassSVM_Self", bound="OneClassSVM")
LinearSVC_Self = TypeVar("LinearSVC_Self", bound="LinearSVC")
LinearSVR_Self = TypeVar("LinearSVR_Self", bound="LinearSVR")


import numpy as np


class LinearSVC(LinearClassifierMixin, SparseCoefMixin, BaseEstimator):
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ndarray = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        penalty: Literal["l1", "l2", "l2"] = "l2",
        loss: Literal["squared_hinge", "hinge", "squared_hinge"] = "squared_hinge",
        *,
        dual: bool = True,
        tol: Float = 1e-4,
        C: Float = 1.0,
        multi_class: Literal["ovr", "crammer_singer", "ovr"] = "ovr",
        fit_intercept: bool = True,
        intercept_scaling: Float = 1,
        class_weight: None | Mapping | str = None,
        verbose: Int = 0,
        random_state: RandomState | None | Int = None,
        max_iter: Int = 1000,
    ) -> None:
        ...

    def fit(
        self: LinearSVC_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> LinearSVC_Self:
        ...


class LinearSVR(RegressorMixin, LinearModel):
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        epsilon: Float = 0.0,
        tol: Float = 1e-4,
        C: Float = 1.0,
        loss: Literal[
            "epsilon_insensitive", "squared_epsilon_insensitive", "epsilon_insensitive"
        ] = "epsilon_insensitive",
        fit_intercept: bool = True,
        intercept_scaling: Float = 1.0,
        dual: bool = True,
        verbose: Int = 0,
        random_state: RandomState | None | Int = None,
        max_iter: Int = 1000,
    ) -> None:
        ...

    def fit(
        self: LinearSVR_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> LinearSVR_Self:
        ...


class SVC(BaseSVC):
    shape_fit_: tuple = ...
    probB_: ndarray = ...
    probA_: ndarray = ...
    n_support_: ndarray = ...
    support_vectors_: ndarray = ...
    support_: ndarray = ...
    n_iter_: ndarray = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercept_: ndarray = ...
    fit_status_: int = ...
    dual_coef_: ndarray = ...
    coef_: ndarray = ...
    classes_: ndarray = ...
    class_weight_: ndarray = ...

    _impl: ClassVar[str] = ...

    def __init__(
        self,
        *,
        C: Float = 1.0,
        kernel: Literal["linear", "poly", "rbf", "sigmoid", "precomputed", "rbf"]
        | Callable = "rbf",
        degree: Int = 3,
        gamma: float | Literal["scale", "auto", "scale"] = "scale",
        coef0: Float = 0.0,
        shrinking: bool = True,
        probability: bool = False,
        tol: Float = 1e-3,
        cache_size: Float = 200,
        class_weight: None | Mapping | str = None,
        verbose: bool = False,
        max_iter: Int = ...,
        decision_function_shape: Literal["ovo", "ovr", "ovr"] = "ovr",
        break_ties: bool = False,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...


class NuSVC(BaseSVC):
    shape_fit_: tuple = ...
    probB_: ndarray = ...
    probA_: ndarray = ...
    n_support_: ndarray = ...
    support_vectors_: ndarray = ...
    support_: ndarray = ...
    n_iter_: ndarray = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercept_: ndarray = ...
    fit_status_: int = ...
    dual_coef_: ndarray = ...
    coef_: ndarray = ...
    classes_: ndarray = ...
    class_weight_: ndarray = ...

    _impl: ClassVar[str] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        nu: Float = 0.5,
        kernel: Literal["linear", "poly", "rbf", "sigmoid", "precomputed", "rbf"]
        | Callable = "rbf",
        degree: Int = 3,
        gamma: float | Literal["scale", "auto", "scale"] = "scale",
        coef0: Float = 0.0,
        shrinking: bool = True,
        probability: bool = False,
        tol: Float = 1e-3,
        cache_size: Float = 200,
        class_weight: None | Mapping | str = None,
        verbose: bool = False,
        max_iter: Int = ...,
        decision_function_shape: Literal["ovo", "ovr", "ovr"] = "ovr",
        break_ties: bool = False,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...


class SVR(RegressorMixin, BaseLibSVM):
    support_vectors_: ndarray = ...
    support_: ndarray = ...
    shape_fit_: tuple = ...
    n_support_: ndarray = ...
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercept_: ndarray = ...
    fit_status_: int = ...
    dual_coef_: ndarray = ...
    coef_: ndarray = ...

    _impl: ClassVar[str] = ...

    _parameter_constraints: ClassVar[dict] = ...
    for unused_param in ["class_weight", "nu", "probability", "random_state"]:
        pass

    def __init__(
        self,
        *,
        kernel: Literal["linear", "poly", "rbf", "sigmoid", "precomputed", "rbf"]
        | Callable = "rbf",
        degree: Int = 3,
        gamma: float | Literal["scale", "auto", "scale"] = "scale",
        coef0: Float = 0.0,
        tol: Float = 1e-3,
        C: Float = 1.0,
        epsilon: Float = 0.1,
        shrinking: bool = True,
        cache_size: Float = 200,
        verbose: bool = False,
        max_iter: Int = ...,
    ) -> None:
        ...

    # TODO(1.4): Remove
    @deprecated(  # type: ignore
        "Attribute `class_weight_` was deprecated in version 1.2 and will be removed in"
        " 1.4."
    )
    @property
    def class_weight_(self) -> ndarray:
        ...


class NuSVR(RegressorMixin, BaseLibSVM):
    support_vectors_: ndarray = ...
    support_: ndarray = ...
    shape_fit_: tuple = ...
    n_support_: ndarray = ...
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercept_: ndarray = ...
    fit_status_: int = ...
    dual_coef_: ndarray = ...
    coef_: ndarray = ...

    _impl: ClassVar[str] = ...

    _parameter_constraints: ClassVar[dict] = ...
    for unused_param in ["class_weight", "epsilon", "probability", "random_state"]:
        pass

    def __init__(
        self,
        *,
        nu: Float = 0.5,
        C: Float = 1.0,
        kernel: Literal["linear", "poly", "rbf", "sigmoid", "precomputed", "rbf"]
        | Callable = "rbf",
        degree: Int = 3,
        gamma: float | Literal["scale", "auto", "scale"] = "scale",
        coef0: Float = 0.0,
        shrinking: bool = True,
        tol: Float = 1e-3,
        cache_size: Float = 200,
        verbose: bool = False,
        max_iter: Int = ...,
    ) -> None:
        ...

    # TODO(1.4): Remove
    @deprecated(  # type: ignore
        "Attribute `class_weight_` was deprecated in version 1.2 and will be removed in"
        " 1.4."
    )
    @property
    def class_weight_(self) -> ndarray:
        ...


class OneClassSVM(OutlierMixin, BaseLibSVM):
    support_vectors_: ndarray = ...
    support_: ndarray = ...
    shape_fit_: tuple = ...
    offset_: float = ...
    n_support_: ndarray = ...
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercept_: ndarray = ...
    fit_status_: int = ...
    dual_coef_: ndarray = ...
    coef_: ndarray = ...

    _impl: ClassVar[str] = ...

    _parameter_constraints: ClassVar[dict] = ...
    for unused_param in ["C", "class_weight", "epsilon", "probability", "random_state"]:
        pass

    def __init__(
        self,
        *,
        kernel: Literal["linear", "poly", "rbf", "sigmoid", "precomputed", "rbf"]
        | Callable = "rbf",
        degree: Int = 3,
        gamma: float | Literal["scale", "auto", "scale"] = "scale",
        coef0: Float = 0.0,
        tol: Float = 1e-3,
        nu: Float = 0.5,
        shrinking: bool = True,
        cache_size: Float = 200,
        verbose: bool = False,
        max_iter: Int = ...,
    ) -> None:
        ...

    # TODO(1.4): Remove
    @deprecated(  # type: ignore
        "Attribute `class_weight_` was deprecated in version 1.2 and will be removed in"
        " 1.4."
    )
    @property
    def class_weight_(self) -> ndarray:
        ...

    def fit(
        self: OneClassSVM_Self,
        X: MatrixLike | ArrayLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> OneClassSVM_Self:
        ...

    def decision_function(self, X: MatrixLike) -> ndarray:
        ...

    def score_samples(self, X: MatrixLike) -> ndarray:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.spatial._hausdorff, version: unspecified
import typing
import builtins as _mod_builtins

__all__: list
__doc__: str
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def directed_hausdorff() -> typing.Any:
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import (
    Literal,
    overload,
)

from pandas import (
    DataFrame,
    Series,
)

from pandas._libs.tslibs import Timedelta
from pandas._typing import (
    AnyArrayLike,
    HashableT,
    JoinHow,
    Label,
    MergeHow,
    ValidationOptions,
)

def merge(
    left: DataFrame | Series,
    right: DataFrame | Series,
    how: MergeHow = ...,
    on: Label | list[HashableT] | AnyArrayLike | None = ...,
    left_on: Label | list[HashableT] | AnyArrayLike | None = ...,
    right_on: Label | list[HashableT] | AnyArrayLike | None = ...,
    left_index: bool = ...,
    right_index: bool = ...,
    sort: bool = ...,
    suffixes: list[str | None]
    | tuple[str, str]
    | tuple[None, str]
    | tuple[str, None] = ...,
    copy: bool = ...,
    indicator: bool | str = ...,
    validate: ValidationOptions = ...,
) -> DataFrame:
    """
Merge DataFrame or named Series objects with a database-style join.

A named Series object is treated as a DataFrame with a single named column.

The join is done on columns or indexes. If joining columns on
columns, the DataFrame indexes *will be ignored*. Otherwise if joining indexes
on indexes or indexes on a column or columns, the index will be passed on.
When performing a cross merge, no column specifications to merge on are
allowed.

.. warning::

    If both key columns contain rows where the key is a null value, those
    rows will be matched against each other. This is different from usual SQL
    join behaviour and can lead to unexpected results.

Parameters
----------
left : DataFrame or named Series
right : DataFrame or named Series
    Object to merge with.
how : {'left', 'right', 'outer', 'inner', 'cross'}, default 'inner'
    Type of merge to be performed.

    * left: use only keys from left frame, similar to a SQL left outer join;
      preserve key order.
    * right: use only keys from right frame, similar to a SQL right outer join;
      preserve key order.
    * outer: use union of keys from both frames, similar to a SQL full outer
      join; sort keys lexicographically.
    * inner: use intersection of keys from both frames, similar to a SQL inner
      join; preserve the order of the left keys.
    * cross: creates the cartesian product from both frames, preserves the order
      of the left keys.

      .. versionadded:: 1.2.0

on : label or list
    Column or index level names to join on. These must be found in both
    DataFrames. If `on` is None and not merging on indexes then this defaults
    to the intersection of the columns in both DataFrames.
left_on : label or list, or array-like
    Column or index level names to join on in the left DataFrame. Can also
    be an array or list of arrays of the length of the left DataFrame.
    These arrays are treated as if they are columns.
right_on : label or list, or array-like
    Column or index level names to join on in the right DataFrame. Can also
    be an array or list of arrays of the length of the right DataFrame.
    These arrays are treated as if they are columns.
left_index : bool, default False
    Use the index from the left DataFrame as the join key(s). If it is a
    MultiIndex, the number of keys in the other DataFrame (either the index
    or a number of columns) must match the number of levels.
right_index : bool, default False
    Use the index from the right DataFrame as the join key. Same caveats as
    left_index.
sort : bool, default False
    Sort the join keys lexicographically in the result DataFrame. If False,
    the order of the join keys depends on the join type (how keyword).
suffixes : list-like, default is ("_x", "_y")
    A length-2 sequence where each element is optionally a string
    indicating the suffix to add to overlapping column names in
    `left` and `right` respectively. Pass a value of `None` instead
    of a string to indicate that the column name from `left` or
    `right` should be left as-is, with no suffix. At least one of the
    values must not be None.
copy : bool, default True
    If False, avoid copy if possible.
indicator : bool or str, default False
    If True, adds a column to the output DataFrame called "_merge" with
    information on the source of each row. The column can be given a different
    name by providing a string argument. The column will have a Categorical
    type with the value of "left_only" for observations whose merge key only
    appears in the left DataFrame, "right_only" for observations
    whose merge key only appears in the right DataFrame, and "both"
    if the observation's merge key is found in both DataFrames.

validate : str, optional
    If specified, checks if merge is of specified type.

    * "one_to_one" or "1:1": check if merge keys are unique in both
      left and right datasets.
    * "one_to_many" or "1:m": check if merge keys are unique in left
      dataset.
    * "many_to_one" or "m:1": check if merge keys are unique in right
      dataset.
    * "many_to_many" or "m:m": allowed, but does not result in checks.

Returns
-------
DataFrame
    A DataFrame of the two merged objects.

See Also
--------
merge_ordered : Merge with optional filling/interpolation.
merge_asof : Merge on nearest keys.
DataFrame.join : Similar method using indices.

Notes
-----
Support for specifying index levels as the `on`, `left_on`, and
`right_on` parameters was added in version 0.23.0
Support for merging named Series objects was added in version 0.24.0

Examples
--------
>>> df1 = pd.DataFrame({'lkey': ['foo', 'bar', 'baz', 'foo'],
...                     'value': [1, 2, 3, 5]})
>>> df2 = pd.DataFrame({'rkey': ['foo', 'bar', 'baz', 'foo'],
...                     'value': [5, 6, 7, 8]})
>>> df1
    lkey value
0   foo      1
1   bar      2
2   baz      3
3   foo      5
>>> df2
    rkey value
0   foo      5
1   bar      6
2   baz      7
3   foo      8

Merge df1 and df2 on the lkey and rkey columns. The value columns have
the default suffixes, _x and _y, appended.

>>> df1.merge(df2, left_on='lkey', right_on='rkey')
  lkey  value_x rkey  value_y
0  foo        1  foo        5
1  foo        1  foo        8
2  foo        5  foo        5
3  foo        5  foo        8
4  bar        2  bar        6
5  baz        3  baz        7

Merge DataFrames df1 and df2 with specified left and right suffixes
appended to any overlapping columns.

>>> df1.merge(df2, left_on='lkey', right_on='rkey',
...           suffixes=('_left', '_right'))
  lkey  value_left rkey  value_right
0  foo           1  foo            5
1  foo           1  foo            8
2  foo           5  foo            5
3  foo           5  foo            8
4  bar           2  bar            6
5  baz           3  baz            7

Merge DataFrames df1 and df2, but raise an exception if the DataFrames have
any overlapping columns.

>>> df1.merge(df2, left_on='lkey', right_on='rkey', suffixes=(False, False))
Traceback (most recent call last):
...
ValueError: columns overlap but no suffix specified:
    Index(['value'], dtype='object')

>>> df1 = pd.DataFrame({'a': ['foo', 'bar'], 'b': [1, 2]})
>>> df2 = pd.DataFrame({'a': ['foo', 'baz'], 'c': [3, 4]})
>>> df1
      a  b
0   foo  1
1   bar  2
>>> df2
      a  c
0   foo  3
1   baz  4

>>> df1.merge(df2, how='inner', on='a')
      a  b  c
0   foo  1  3

>>> df1.merge(df2, how='left', on='a')
      a  b  c
0   foo  1  3.0
1   bar  2  NaN

>>> df1 = pd.DataFrame({'left': ['foo', 'bar']})
>>> df2 = pd.DataFrame({'right': [7, 8]})
>>> df1
    left
0   foo
1   bar
>>> df2
    right
0   7
1   8

>>> df1.merge(df2, how='cross')
   left  right
0   foo      7
1   foo      8
2   bar      7
3   bar      8
    """
    pass
@overload
def merge_ordered(
    left: DataFrame,
    right: DataFrame,
    on: Label | list[HashableT] | None = ...,
    left_on: Label | list[HashableT] | None = ...,
    right_on: Label | list[HashableT] | None = ...,
    left_by: Label | list[HashableT] | None = ...,
    right_by: Label | list[HashableT] | None = ...,
    fill_method: Literal["ffill"] | None = ...,
    suffixes: list[str | None]
    | tuple[str, str]
    | tuple[None, str]
    | tuple[str, None] = ...,
    how: JoinHow = ...,
) -> DataFrame: ...
@overload
def merge_ordered(
    left: Series,
    right: DataFrame | Series,
    on: Label | list[HashableT] | None = ...,
    left_on: Label | list[HashableT] | None = ...,
    right_on: Label | list[HashableT] | None = ...,
    left_by: None = ...,
    right_by: None = ...,
    fill_method: Literal["ffill"] | None = ...,
    suffixes: list[str | None]
    | tuple[str, str]
    | tuple[None, str]
    | tuple[str, None] = ...,
    how: JoinHow = ...,
) -> DataFrame: ...
@overload
def merge_ordered(
    left: DataFrame | Series,
    right: Series,
    on: Label | list[HashableT] | None = ...,
    left_on: Label | list[HashableT] | None = ...,
    right_on: Label | list[HashableT] | None = ...,
    left_by: None = ...,
    right_by: None = ...,
    fill_method: Literal["ffill"] | None = ...,
    suffixes: list[str | None]
    | tuple[str, str]
    | tuple[None, str]
    | tuple[str, None] = ...,
    how: JoinHow = ...,
) -> DataFrame: ...
def merge_asof(
    left: DataFrame | Series,
    right: DataFrame | Series,
    on: Label | None = ...,
    left_on: Label | None = ...,
    right_on: Label | None = ...,
    left_index: bool = ...,
    right_index: bool = ...,
    by: Label | list[HashableT] | None = ...,
    left_by: Label | list[HashableT] | None = ...,
    right_by: Label | list[HashableT] | None = ...,
    suffixes: list[str | None]
    | tuple[str, str]
    | tuple[None, str]
    | tuple[str, None] = ...,
    tolerance: int | Timedelta | None = ...,
    allow_exact_matches: bool = ...,
    direction: Literal["backward", "forward", "nearest"] = ...,
) -> DataFrame: ...
